@@ -4,7 +4,7 @@ import os
 from sys import path
 from typing import Any, Dict, List
 
-from anyscale import snapshot_util, version
+from anyscale import version
 
 
 logger = logging.getLogger(__name__)
@@ -15,8 +15,9 @@ anyscale_dir = os.path.dirname(os.path.abspath(__file__))
 path.append(os.path.join(anyscale_dir, "client"))
 path.append(os.path.join(anyscale_dir, "sdk"))
 
-from anyscale import integrations
+from anyscale import compute_config, integrations, job, service
 from anyscale.cluster import get_job_submission_client_cluster_info
+from anyscale.cluster_compute import get_cluster_compute_from_name
 from anyscale.connect import ClientBuilder
 from anyscale.sdk.anyscale_client.sdk import AnyscaleSDK
 
