@@ -104,7 +104,9 @@ class BlockLogger:
     def zero_time(self) -> None:
         self.t0 = time.time()
 
-    def info(self, *msg: str, block_label: Optional[str] = None) -> None:
+    def info(
+        self, *msg: str, block_label: Optional[str] = None, end: str = "\n"
+    ) -> None:
         if not self.log_output:
             return
         if block_label:
@@ -129,7 +131,7 @@ class BlockLogger:
             )
             print(INDENT * self.indent_level, end="", file=sys.stderr)
             print(
-                *msg, file=sys.stderr,
+                *msg, file=sys.stderr, end=end,
             )
 
     def debug(self, *msg: str) -> None:
@@ -255,6 +257,23 @@ class LogsLogger(BlockLogger):
             yield spinner
 
 
+class StringLogger(LogsLogger):
+    """ This logger is used to print customer logs to a string with no decoration
+    """
+
+    def __init__(self):
+        super().__init__(False)
+        self.out_string = ""
+
+    def log(self, msg: str):
+        self.out_string += f"{msg}\n"
+
+    def is_interactive_cli_enabled(self) -> bool:
+        """Check if shell is interactive
+        """
+        return False
+
+
 class CloudSetupLogger(LogsLogger):
     def __init__(
         self,
