@@ -134,15 +134,18 @@ def list_builds(
         anyscale_api_client = get_auth_api_client().anyscale_api_client
     entities: List[ClusterEnvironmentBuild] = []
     has_more = (len(entities) < max_items) if max_items else True
-    while has_more:
+    paging_token = None
+    i = 0
+    while has_more and i < 100:
         resp = anyscale_api_client.list_cluster_environment_builds(
-            cluster_env_id, count=50, paging_token=None
+            cluster_env_id, count=50, paging_token=paging_token
         )
         entities.extend(resp.results)
         paging_token = resp.metadata.next_paging_token
         has_more = paging_token is not None
         if max_items:
             has_more = has_more and (len(entities) < max_items)
+        i += 1
     if max_items:
         return entities[:max_items]
     else:
