@@ -8,6 +8,7 @@ from anyscale.client.openapi_client.models import ClusterManagementStackVersions
 from anyscale.commands import cloud_commands_util
 from anyscale.commands.util import OptionPromptNull
 from anyscale.controllers.cloud_controller import CloudController
+from anyscale.util import validate_non_negative_arg
 
 
 log = BlockLogger()  # CLI Logger
@@ -124,6 +125,16 @@ def cloud_set_default(
 @click.option(
     "--yes", "-y", is_flag=True, default=False, help="Skip asking for confirmation."
 )
+@click.option(
+    "--disable-auto-add-user",
+    is_flag=True,
+    default=False,
+    help=(
+        "All users in the organization will be added to clouds created "
+        "with `anyscale cloud setup` by default. Specify --disable-auto-add-user to "
+        "disable this and instead manually grant users permissions to the cloud."
+    ),
+)
 def setup_cloud(  # noqa: PLR0913
     provider: str,
     region: str,
@@ -133,6 +144,7 @@ def setup_cloud(  # noqa: PLR0913
     anyscale_managed: bool,  # noqa: ARG001
     enable_head_node_fault_tolerance: bool,
     yes: bool,
+    disable_auto_add_user: bool,
 ) -> None:
     # TODO (congding): remove `anyscale_managed` in the future, now keeping it for compatibility
     if provider == "aws":
@@ -144,10 +156,16 @@ def setup_cloud(  # noqa: PLR0913
             cluster_management_stack_version=ClusterManagementStackVersions.V2,
             enable_head_node_fault_tolerance=enable_head_node_fault_tolerance,
             yes=yes,
+            auto_add_user=(not disable_auto_add_user),
         )
     elif provider == "gcp":
         if not project_id:
             project_id = click.prompt("GCP Project ID", type=str)
+        if project_id[0].isdigit():
+            # project ID should start with a letter
+            raise click.ClickException(
+                "Please provide a valid project ID. Note that project ID is not project number, see https://cloud.google.com/resource-manager/docs/creating-managing-projects#before_you_begin for details."
+            )
         CloudController().setup_managed_cloud(
             provider=provider,
             region=region,
@@ -157,6 +175,7 @@ def setup_cloud(  # noqa: PLR0913
             cluster_management_stack_version=ClusterManagementStackVersions.V2,
             enable_head_node_fault_tolerance=enable_head_node_fault_tolerance,
             yes=yes,
+            auto_add_user=(not disable_auto_add_user),
         )
 
 
@@ -177,8 +196,20 @@ def setup_cloud(  # noqa: PLR0913
     default=None,
     help=("Id of cloud to get information about."),
 )
-def list_cloud(name: Optional[str], cloud_id: Optional[str],) -> None:
-    print(CloudController().list_clouds(cloud_name=name, cloud_id=cloud_id,))
+@click.option(
+    "--max-items",
+    required=False,
+    default=20,
+    type=int,
+    help="Max items to show in list.",
+    callback=validate_non_negative_arg,
+)
+def list_cloud(name: Optional[str], cloud_id: Optional[str], max_items: int,) -> None:
+    print(
+        CloudController().list_clouds(
+            cloud_name=name, cloud_id=cloud_id, max_items=max_items
+        )
+    )
 
 
 @cloud_cli.group("config", help="Manage the configuration for a cloud.")
@@ -213,6 +244,17 @@ def cloud_config_group() -> None:
     default=False,
     help="Whether to enable head node fault tolerance for services.",
 )
+@click.option(
+    "--enable-auto-add-user/--disable-auto-add-user",
+    default=None,
+    help=(
+        "If --enable-auto-add-user is specified for a cloud, all users in the organization "
+        "will be added to the cloud by default. Note: There may be up to 30 sec delay for all users to be granted "
+        "permissions after this feature is enabled.\n\n"
+        "Specifying --disable-auto-add-user will require that users "
+        "are manually granted permissions to access the cloud. No existing cloud permissions are altered by specifying this flag."
+    ),
+)
 def cloud_update(
     cloud_name: Optional[str],
     name: Optional[str],
@@ -220,18 +262,27 @@ def cloud_update(
     functional_verify: Optional[str],
     enable_head_node_fault_tolerance: bool,
     yes: bool,
+    enable_auto_add_user: Optional[bool],
 ) -> None:
     if cloud_name and name and cloud_name != name:
         raise click.ClickException(
             "The positional argument CLOUD_NAME and the keyword argument --name "
             "were both provided. Please only provide one of these two arguments."
         )
+    if enable_head_node_fault_tolerance and (enable_auto_add_user is not None):
+        raise click.ClickException(
+            "Please only specify either --enable-head-node-fault-tolerance or "
+            f"{'--enable-auto-add-user' if enable_auto_add_user else '--disable-auto-add-user'} for "
+            "this call of `anyscale cloud update`. The other flag can be specified in a separate call "
+            "to the command."
+        )
     CloudController().update_managed_cloud(
         cloud_name=cloud_name or name,
         cloud_id=cloud_id,
         enable_head_node_fault_tolerance=enable_head_node_fault_tolerance,
         functional_verify=functional_verify,
         yes=yes,
+        auto_add_user=enable_auto_add_user,
     )
 
 
@@ -379,6 +430,12 @@ def cloud_config_get(
     required=False,
     type=str,
 )
+@click.option(
+    "--host-project-id",
+    help="Host project ID for shared VPC",
+    required=False,
+    type=str,
+)
 @click.option(
     "--private-network", help="Use private network.", is_flag=True, default=False,
 )
@@ -392,7 +449,26 @@ def cloud_config_get(
 @click.option(
     "--yes", "-y", is_flag=True, default=False, help="Skip asking for confirmation."
 )
-def register_cloud(  # noqa: PLR0913
+@click.option(
+    "--skip-verifications",
+    help="Skip verifications. This will skip all verifications.",
+    required=False,
+    is_flag=True,
+    type=bool,
+    default=False,
+)
+@click.option(
+    "--enable-auto-add-user",
+    is_flag=True,
+    default=False,
+    help=(
+        "If --enable-auto-add-user is specified for a cloud, all users in the organization "
+        "will be added to the cloud by default. Otherwise users will need to be manually granted "
+        "permissions to the cloud. Note: There may be up to 30 sec delay for all users to be granted "
+        "permissions after the cloud is created."
+    ),
+)
+def register_cloud(  # noqa: PLR0913, PLR0912
     provider: str,
     region: str,
     name: str,
@@ -415,9 +491,12 @@ def register_cloud(  # noqa: PLR0913
     firewall_policy_names: str,
     cloud_storage_bucket_name: str,
     memorystore_instance_name: str,
+    host_project_id: Optional[str],
     functional_verify: Optional[str],
     private_network: bool,
     yes: bool,
+    skip_verifications: bool,
+    enable_auto_add_user: bool,
 ) -> None:
     missing_args: List[str] = []
     if provider == "aws":
@@ -453,11 +532,13 @@ def register_cloud(  # noqa: PLR0913
             private_network=private_network,
             cluster_management_stack_version=ClusterManagementStackVersions.V2,
             yes=yes,
+            skip_verifications=skip_verifications,
+            auto_add_user=enable_auto_add_user,
         )
     elif provider == "gcp":
         # Keep the parameter naming ({resource}_name or {resource}_id) consistent with GCP to reduce confusion for customers
         # Check if all required parameters are provided
-        # Only memorystore_instance_name is optional for GCP clouds
+        # memorystore_instance_name and host_project_id are optional for GCP clouds
         for resource in [
             (project_id, "--project-id"),
             (vpc_name, "--vpc-name"),
@@ -503,6 +584,12 @@ def register_cloud(  # noqa: PLR0913
                 "Please provide a valid memorystore instance name. Example: projects/<project number>/locations/<location>/instances/<instance id>"
             )
 
+        if host_project_id is not None and host_project_id[0].isdigit():
+            # project ID should start with a letter
+            raise click.ClickException(
+                "Please provide a valid project ID for `--host-project-id`. Note that project ID is not project number, see https://cloud.google.com/resource-manager/docs/creating-managing-projects#before_you_begin for details."
+            )
+
         CloudController().register_gcp_cloud(
             region=region,
             name=name,
@@ -518,10 +605,13 @@ def register_cloud(  # noqa: PLR0913
             firewall_policy_names=firewall_policy_names.split(","),
             cloud_storage_bucket_name=cloud_storage_bucket_name,
             memorystore_instance_name=memorystore_instance_name,
+            host_project_id=host_project_id,
             functional_verify=functional_verify,
             private_network=private_network,
             cluster_management_stack_version=ClusterManagementStackVersions.V2,
             yes=yes,
+            skip_verifications=skip_verifications,
+            auto_add_user=enable_auto_add_user,
         )
     else:
         raise click.ClickException(
@@ -628,6 +718,17 @@ def cloud_verify(
     is_flag=False,
     flag_value="workspace",
 )
+@click.option(
+    "--enable-auto-add-user/--disable-auto-add-user",
+    default=None,
+    help=(
+        "If --enable-auto-add-user is specified for a cloud, all users in the organization "
+        "will be added to the cloud by default. Note: There may be up to 30 sec delay for all users to be granted "
+        "permissions after this feature is enabled.\n\n"
+        "Specifying --disable-auto-add-user will require that users "
+        "are manually granted permissions to access the cloud. No existing cloud permissions are altered by specifying this flag."
+    ),
+)
 def cloud_edit(  # noqa: PLR0913
     cloud_name: Optional[str],
     name: Optional[str],
@@ -641,6 +742,7 @@ def cloud_edit(  # noqa: PLR0913
     gcp_cloud_storage_bucket_name: Optional[str],
     memorystore_instance_name: Optional[str],
     functional_verify: Optional[str],
+    enable_auto_add_user: Optional[bool],
 ) -> None:
     if cloud_name and name and cloud_name != name:
         raise click.ClickException(
@@ -657,6 +759,7 @@ def cloud_edit(  # noqa: PLR0913
             gcp_filestore_location,
             gcp_cloud_storage_bucket_name,
             memorystore_instance_name,
+            enable_auto_add_user is not None,
         ]
     ):
         if any([gcp_filestore_instance_id, gcp_filestore_location]) and not all(
@@ -687,6 +790,7 @@ def cloud_edit(  # noqa: PLR0913
             gcp_cloud_storage_bucket_name=gcp_cloud_storage_bucket_name,
             memorystore_instance_name=memorystore_instance_name,
             functional_verify=functional_verify,
+            auto_add_user=enable_auto_add_user,
         )
     else:
         raise click.ClickException(
