@@ -208,10 +208,16 @@ def list(  # noqa: A001
     default=None,
     help=("Get details about compute configuration by this id."),
 )
+@click.option(
+    "--include-archived",
+    is_flag=True,
+    help=("Include archived compute configurations."),
+)
 def get(
     compute_config_name: Optional[str],
     cluster_compute_id: Optional[str],
     compute_config_id: Optional[str],
+    include_archived: bool,
 ) -> None:
     cluster_compute_controller = ComputeConfigController()
 
@@ -223,4 +229,5 @@ def get(
     cluster_compute_controller.get(
         cluster_compute_name=compute_config_name,
         cluster_compute_id=compute_config_id or cluster_compute_id,
+        include_archived=include_archived,
     )
