@@ -7,6 +7,7 @@ import click
 import anyscale
 from anyscale.cli_logger import BlockLogger
 from anyscale.client.openapi_client import HaJobStates
+from anyscale.commands.util import convert_kv_strings_to_dict
 from anyscale.controllers.job_controller import JobController
 from anyscale.job.models import JobConfig
 from anyscale.util import validate_non_negative_arg
@@ -23,70 +24,186 @@ def job_cli() -> None:
 @job_cli.command(
     name="submit", short_help="Submit a job.",
 )
-@click.option("--name", "-n", required=False, default=None, help="Name of the job.")
+@click.option("-n", "--name", required=False, default=None, help="Name of the job.")
 @click.option(
-    "--description", required=False, default=None, help="Description of the job."
-)
-@click.option(
-    "--follow",
     "-f",
+    "--follow",
     required=False,
     default=False,
     type=bool,
     is_flag=True,
-    help="Whether to follow the log of the created job",
+    help="DEPRECATED: use `--wait` instead.",
 )
 @click.option(
-    "--wait",
     "-w",
+    "--wait",
     required=False,
     default=False,
     type=bool,
     is_flag=True,
-    help="Whether to block this CLI command until the job succeeds (or reaches another terminal state)",
+    help="Block this CLI command and print logs until the job finishes.",
+)
+@click.option(
+    "--config-file",
+    required=False,
+    default=None,
+    type=str,
+    help="Path to a YAML config file to use for this job. Command-line flags will overwrite values read from the file.",
+)
+@click.option(
+    "--image-uri",
+    required=False,
+    default=None,
+    type=str,
+    help="Container image to use for this job. When running in a workspace, this defaults to the image of the workspace.",
+)
+@click.option(
+    "--containerfile",
+    required=False,
+    default=None,
+    type=str,
+    help="Path to a containerfile to build the image to use for the job. This is exclusive with --image-uri.",
+)
+@click.option(
+    "-e",
+    "--env",
+    required=False,
+    multiple=True,
+    type=str,
+    help="Environment variables to set for the job. The format is 'key=value'. This argument can be specified multiple times. When the same key is also specified in the config file, the value from the command-line flag will overwrite the value from the config file.",
+)
+@click.option(
+    "--working-dir",
+    required=False,
+    default=None,
+    type=str,
+    help="Path to a local directory that will be the working directory for the job. The files in the directory will be automatically uploaded to cloud storage. When running in a workspace, this defaults to the current working directory.",
+)
+@click.option(
+    "--py-module",
+    required=False,
+    default=None,
+    multiple=True,
+    type=str,
+    help="Python modules to be available for import in the Ray workers. Each entry must be a path to a local directory.",
 )
 @click.argument("entrypoint", required=False, nargs=-1, type=click.UNPROCESSED)
-def submit(
+def submit(  # noqa: PLR0912 PLR0913 C901
     entrypoint: Tuple[str],
     name: Optional[str],
-    description: Optional[str],
     follow: Optional[bool],
     wait: Optional[bool],
+    config_file: Optional[str],
+    image_uri: Optional[str],
+    containerfile: Optional[str],
+    env: Tuple[str],
+    working_dir: Optional[str],
+    py_module: Tuple[str],
 ):
     """Submit a job.
 
-    The job config can be specified from a file or inlined in this command:
+    The job config can be specified in one of the following ways:
 
-    $ anyscale job submit config.yaml # Submit from a config file.
+    * Job config file can be specified as a single positional argument. E.g. `anyscale job submit config.yaml`.
 
-    $ anyscale job submit -- python main.py # Submit from an inlined entrypoint.
+    * Job config can also be specified with command-line arguments. In this case, the entrypoint should be specified
+as the positional arguments starting with `--`. Other arguments can be specified with command-line flags. E.g.
+
+      * `anyscale job submit -- python main.py`: submit a job with the entrypoint `python main.py`.
+
+      * `anyscale job submit --name my-job -- python main.py`: submit a job with the name `my-job` and the
+entrypoint `python main.py`.
+
+    * [Experimental] If you want to specify a config file and override some arguments with the commmand-line flags,
+use the `--config-file` flag. E.g.
+
+      * `anyscale job submit --config-file config.yaml`: submit a job with the config in `config.yaml`.
+
+      * `anyscale job submit --config-file config.yaml -- python main.py`: submit a job with the config in `config.yaml`
+and override the entrypoint with `python main.py`.
 
     By default, this command submits the job asynchronously and exits. To wait for the job to complete, use the `--wait` flag.
     """
 
-    if len(entrypoint) == 0:
-        raise click.ClickException(
-            "Either a config file or an inlined entrypoint must be provided."
-        )
-
-    config_path: Optional[str] = None
-    if len(entrypoint) == 1:
-        maybe_config_path = entrypoint[0]
-        if pathlib.Path(maybe_config_path).is_file():
-            config_path = entrypoint[0]
-            log.info(f"Submitting job from config file: '{config_path}'.")
-        elif maybe_config_path.endswith(".yaml"):
+    job_controller = JobController()
+    if len(entrypoint) == 1 and (
+        pathlib.Path(entrypoint[0]).is_file() or entrypoint[0].endswith(".yaml")
+    ):
+        # If entrypoint is a single string that ends with .yaml, e.g. `anyscale job submit config.yaml`,
+        # treat it as a config file, and use the old job submission API.
+        if config_file is not None:
             raise click.ClickException(
-                f"Job config file '{maybe_config_path}' not found."
+                "`--config-file` should not be used when providing a config file as the entrypoint."
+            )
+        if image_uri:
+            raise click.ClickException(
+                "`--image-uri` should not be used when providing a config file as the entrypoint."
+            )
+        if containerfile:
+            raise click.ClickException(
+                "`--containerfile` should not be used when providing a config file as the entrypoint."
+            )
+        if env:
+            raise click.ClickException(
+                "`--env` should not be used when providing a config file as the entrypoint."
             )
 
-    job_controller = JobController()
-    if config_path is not None:
-        job_id = job_controller.submit(config_path, name=name, description=description,)
+        config_file = entrypoint[0]
+        if not pathlib.Path(config_file).is_file():
+            raise click.ClickException(f"Job config file '{config_file}' not found.")
+        log.info(f"Submitting job from config file {config_file}.")
+
+        job_id = job_controller.submit(config_file, name=name)
     else:
-        job_id = anyscale.job.submit(
-            JobConfig(name=name, entrypoint=list2cmdline(entrypoint))
-        )
+        # Otherwise, use the new job submission API. E.g.
+        # `anyscale job submit -- python main.py`,
+        # or `anyscale job submit --config-file config.yaml`.
+        if len(entrypoint) == 0 and config_file is None:
+            raise click.ClickException(
+                "Either a config file or an inlined entrypoint must be provided."
+            )
+        if config_file is not None and not pathlib.Path(config_file).is_file():
+            raise click.ClickException(f"Job config file '{config_file}' not found.")
+        if follow:
+            log.warning("`--follow` is deprecated, use `--wait` instead.")
+
+        args = {}
+        if len(entrypoint) > 0:
+            args["entrypoint"] = list2cmdline(entrypoint)
+        if name:
+            args["name"] = name
+
+        if containerfile and image_uri:
+            raise click.ClickException(
+                "Only one of '--containerfile' and '--image-uri' can be provided."
+            )
+
+        if image_uri:
+            args["image_uri"] = image_uri
+
+        if containerfile:
+            args["containerfile"] = containerfile
+
+        if working_dir:
+            args["working_dir"] = working_dir
+
+        if config_file is not None:
+            config = JobConfig.from_yaml(config_file, **args)
+        else:
+            config = JobConfig.from_dict(args)
+        if env:
+            env_dict = convert_kv_strings_to_dict(env)
+            if env_dict:
+                config = config.options(env_vars=env_dict)
+        if py_module:
+            for module in py_module:
+                if not pathlib.Path(module).is_dir():
+                    raise click.ClickException(
+                        f"Python module path '{module}' does not exist or is not a directory."
+                    )
+            config = config.options(py_modules=[*py_module])
+        log.info(f"Submitting job with config {config}.")
+        job_id = anyscale.job.submit(config)
 
     if follow or wait:
         job_controller.log.info(
