@@ -1,4 +1,5 @@
 import contextlib
+import os
 import time
 import webbrowser
 
@@ -16,6 +17,7 @@ log = BlockLogger()
 
 def get_unauthenticated_openapi_client():
     conf = openapi_client.Configuration(host=ANYSCALE_HOST)
+    conf.proxy = os.environ.get("https_proxy")
     conf.connection_pool_maxsize = 100
     return openapi_client.DefaultApi(ApiClientWrapperInternal(conf))
 
