@@ -8,6 +8,7 @@ from typing_extensions import Literal
 import yaml
 
 from anyscale.cli_logger import BlockLogger
+from anyscale.commands.util import convert_kv_strings_to_dict
 from anyscale.controllers.service_controller import ServiceController
 import anyscale.service
 from anyscale.service import ServiceConfig, ServiceStatus
@@ -22,26 +23,6 @@ def service_cli():
     pass
 
 
-def _convert_arguments_to_dict(args: Tuple[str]) -> Dict[str, str]:
-    """Convert args of the form "key=val" into a dictionary of {key: val}.
-
-    NOTE(edoakes): this mimics the functionality of the `serve run` CLI and should be
-    kept in sync with it.
-    """
-    args_dict = {}
-    for arg in args:
-        split = arg.split("=")
-        if len(split) != 2:
-            raise click.ClickException(
-                f"Invalid application argument '{arg}', "
-                "must be of the form '<key>=<val>'."
-            )
-
-        args_dict[split[0]] = split[1]
-
-    return args_dict
-
-
 def _read_name_from_config_file(path: str):
     """Read the 'name' property from the config file at `path`.
 
@@ -86,7 +67,7 @@ def _read_name_from_config_file(path: str):
     required=False,
     default=None,
     type=str,
-    help="Container image to use for the service. This cannot be changed when using --in-place. When running in a workspace, this defaults to the image of the workspace.",
+    help="Container image to use for the service. This cannot be changed when using --in-place and is exclusive with --containerfile. When running in a workspace, this defaults to the image of the workspace.",
 )
 @click.option(
     "--containerfile",
@@ -132,7 +113,7 @@ def _read_name_from_config_file(path: str):
     is_flag=True,
     show_default=True,
     default=False,
-    help="Perform an in-place upgrade without starting a new cluster. This can be used for faster iteration during development but is *not* currently recommended for production deploys.",
+    help="Perform an in-place upgrade without starting a new cluster. This can be used for faster iteration during development but is *not* currently recommended for production deploys. This *cannot* be used to change cluster-level options such as image and compute config (they will be ignored).",
 )
 @click.option(
     "--canary-percent",
@@ -148,7 +129,22 @@ def _read_name_from_config_file(path: str):
     type=int,
     help="Amount of excess capacity allowed to be used while updating the service (0-100). Defaults to 100. Not supported when using --in-place.",
 )
-def deploy(  # noqa: PLR0912, PLR0913
+@click.option(
+    "--env",
+    required=False,
+    multiple=True,
+    type=str,
+    help="Environment variables to set for the service. The format is 'key=value'. This argument can be specified multiple times. When the same key is also specified in the config file, the value from the command-line flag will overwrite the value from the config file.",
+)
+@click.option(
+    "--py-module",
+    required=False,
+    default=None,
+    multiple=True,
+    type=str,
+    help="Python modules to be available for import in the Ray workers. Each entry must be a path to a local directory.",
+)
+def deploy(  # noqa: PLR0912, PLR0913 C901
     config_file: Optional[str],
     import_path: Optional[str],
     arguments: Tuple[str],
@@ -162,6 +158,8 @@ def deploy(  # noqa: PLR0912, PLR0913
     in_place: bool,
     canary_percent: Optional[int],
     max_surge_percent: Optional[int],
+    env: Optional[Tuple[str]],
+    py_module: Tuple[str],
 ):
     """Deploy or update a service.
 
@@ -169,11 +167,11 @@ def deploy(  # noqa: PLR0912, PLR0913
 
     To deploy a single application, the import path and optional arguments can be passed directly as arguments:
 
-    $ anyscale service deploy main:app arg1=val1
+    `$ anyscale service deploy main:app arg1=val1`
 
     To deploy multiple applications, deploy from a config file using `-f`:
-t
-    $ anyscale service deploy -f config.yaml
+
+    `$ anyscale service deploy -f config.yaml`
 
     Command-line flags override values in the config file.
     """
@@ -203,9 +201,9 @@ t
             )
 
         app: Dict[str, Any] = {"import_path": import_path}
-        arguments_dict = _convert_arguments_to_dict(arguments)
+        arguments_dict = convert_kv_strings_to_dict(arguments)
         if arguments_dict:
-            app["arguments"] = arguments_dict
+            app["args"] = arguments_dict
 
         config = ServiceConfig(applications=[app])
 
@@ -235,6 +233,19 @@ t
     if requirements is not None:
         config = config.options(requirements=requirements)
 
+    if env:
+        env_dict = convert_kv_strings_to_dict(env)
+        if env_dict:
+            config = config.options(env_vars=env_dict)
+
+    if py_module:
+        for module in py_module:
+            if not pathlib.Path(module).is_dir():
+                raise click.ClickException(
+                    f"Python module path '{module}' does not exist or is not a directory."
+                )
+        config = config.options(py_modules=[*py_module])
+
     anyscale.service.deploy(
         config,
         in_place=in_place,
@@ -252,7 +263,7 @@ t
     required=False,
     default=None,
     type=str,
-    help="Unique name for the service. When running in a workspace, this defaults to the workspace name.",
+    help="Name of the service. When running in a workspace, this defaults to the workspace name.",
 )
 @click.option(
     "-f",
@@ -263,15 +274,15 @@ t
     help="Path to a YAML config file to read the name from.",
 )
 @click.option(
-    "--json",
     "-j",
+    "--json",
     is_flag=True,
     default=False,
     help="Output the status in a structured JSON format.",
 )
 @click.option(
-    "--verbose",
     "-v",
+    "--verbose",
     is_flag=True,
     default=False,
     help="Include verbose details in the status.",
@@ -290,8 +301,6 @@ def status(name: Optional[str], config_file: Optional[str], json: bool, verbose:
     if not verbose:
         # TODO(edoakes): consider adding this as an API on the model itself if it
         # becomes a common pattern.
-        status_dict.pop("query_url", None)
-        status_dict.pop("query_auth_token", None)
         status_dict.get("primary_version", {}).pop("config", None)
         status_dict.get("canary_version", {}).pop("config", None)
 
@@ -308,6 +317,7 @@ def status(name: Optional[str], config_file: Optional[str], json: bool, verbose:
 )
 @click.option(
     "-f",
+    "--config-file",
     "--service-config-file",
     required=True,
     help="The path of the service configuration file.",
@@ -352,7 +362,7 @@ def status(name: Optional[str], config_file: Optional[str], json: bool, verbose:
     help="Do not complete the rollout (terminate the existing version cluster) after the canary version reaches 100%",
 )
 def rollout(  # noqa: PLR0913
-    service_config_file: str,
+    config_file: str,
     name: Optional[str],
     version: Optional[str],
     max_surge_percent: Optional[int],
@@ -370,7 +380,7 @@ def rollout(  # noqa: PLR0913
         rollout_strategy = "IN_PLACE"
 
     service_controller = ServiceController()
-    service_config = service_controller.read_service_config_file(service_config_file)
+    service_config = service_controller.read_service_config_file(config_file)
     service_controller.rollout(
         service_config,
         name=name,
@@ -431,10 +441,13 @@ def list(  # noqa: A001
 @click.option(
     "--service-id", "--id", default=None, help="ID of service.",
 )
-@click.option("--name", "-n", required=False, default=None, help="Name of service.")
+@click.option("-n", "--name", required=False, default=None, help="Name of service.")
 @click.option("--project-id", required=False, help="Filter by project id.")
 @click.option(
-    "--service-config-file", "-f", help="The path of the service configuration file.",
+    "-f",
+    "--config-file",
+    "--service-config-file",
+    help="Path to a YAML config file to read the name from. `--service-config-file` is deprecated, use `-f` or `--config-file`.",
 )
 @click.option(
     "--max-surge-percent",
@@ -447,7 +460,7 @@ def rollback(
     service_id: Optional[str],
     name: Optional[str],
     project_id: Optional[str],
-    service_config_file: Optional[str],
+    config_file: Optional[str],
     max_surge_percent: Optional[int],
 ):
     """Perform a rollback for a service that is currently in a rollout.
@@ -458,7 +471,7 @@ def rollback(
     service_id = service_controller.get_service_id(
         service_id=service_id,
         service_name=name,
-        service_config_file=service_config_file,
+        service_config_file=config_file,
         project_id=project_id,
     )
     service_controller.rollback(service_id, max_surge_percent)
@@ -468,16 +481,19 @@ def rollback(
 @click.option(
     "--service-id", "--id", required=False, help="ID of service.",
 )
-@click.option("--name", "-n", required=False, help="Name of service.")
+@click.option("-n", "--name", required=False, help="Name of service.")
 @click.option("--project-id", required=False, help="Filter by project id.")
 @click.option(
-    "--service-config-file", "-f", help="The path of the service configuration file.",
+    "-f",
+    "--config-file",
+    "--service-config-file",
+    help="Path to a YAML config file to read the name from. `--service-config-file` is deprecated, use `-f` or `--config-file`.",
 )
 def terminate(
     service_id: Optional[str],
     name: Optional[str],
     project_id: Optional[str],
-    service_config_file: Optional[str],
+    config_file: Optional[str],
 ):
     """Terminate a service.
 
@@ -487,7 +503,7 @@ def terminate(
     service_id = service_controller.get_service_id(
         service_id=service_id,
         service_name=name,
-        service_config_file=service_config_file,
+        service_config_file=config_file,
         project_id=project_id,
     )
     service_controller.terminate(service_id)
