@@ -1,3 +1,5 @@
+from typing import Dict, Tuple
+
 import click
 
 
@@ -58,3 +60,22 @@ class NotRequiredIf(click.Option):
                 self.default = None
 
         return super().handle_parse_result(ctx, opts, args)
+
+
+def convert_kv_strings_to_dict(strings: Tuple[str]) -> Dict[str, str]:
+    """Convert args/env_vars of the form "key=val" into a dictionary of {key: val}.
+
+    NOTE(edoakes): this mimics the functionality of the `serve run` CLI and should be
+    kept in sync with it.
+    """
+    ret_dict = {}
+    for s in strings:
+        split = s.split("=")
+        if len(split) != 2:
+            raise click.ClickException(
+                f"Invalid key-value string '{s}'. Must be of the form 'key=value'."
+            )
+
+        ret_dict[split[0]] = split[1]
+
+    return ret_dict
