@@ -342,8 +342,7 @@ def _do_pull(pull_git_state):
     # functional locally. A repack is expensive, but we assume pulls aren't frequent.
     if pull_git_state:
         workspace_controller.run_cmd(
-            f'cd ~/{dir_name} && [[ -n "$ENABLE_MOUNT_SNAPSHOT_UTIL" ]] && python -m snapshot_util repack_git_repos || git repack -a -d -q',
-            args=[],
+            f"cd ~/{dir_name} && python -m snapshot_util repack_git_repos", args=[],
         )
 
     workspace_root = get_workspace_root_or_throw()
