@@ -10,12 +10,14 @@ from anyscale.authenticate import get_auth_api_client
 from anyscale.cli_logger import BlockLogger
 from anyscale.client.openapi_client.models.build import Build
 from anyscale.client.openapi_client.models.session import Session
-from anyscale.cluster_compute import get_selected_cloud_id_or_default
+from anyscale.cluster_compute import (
+    get_cluster_compute_from_name,
+    get_selected_cloud_id_or_default,
+)
 from anyscale.conf import MINIMUM_RAY_VERSION
 from anyscale.links import DOCS_CLUSTER
 from anyscale.sdk.anyscale_client import (
     ComputeTemplateConfig,
-    ComputeTemplateQuery,
     CreateCluster,
     CreateComputeTemplate,
     StartClusterOptions,
@@ -89,7 +91,6 @@ class PrepareClusterBlock:
         cluster_env_revision: Optional[int],
         ray: Any,
         log_output: bool = True,
-        cluster_compute_version: Optional[int] = None,
     ):
         """
         Class to prepare a cluster (create and start if required) so it can be
@@ -102,7 +103,6 @@ class PrepareClusterBlock:
         self.allow_public_internet_traffic = allow_public_internet_traffic
         self.needs_update = needs_update
         self.cluster_compute_name = cluster_compute_name
-        self.cluster_compute_version = cluster_compute_version
         self.cluster_compute_dict = cluster_compute_dict
         self.cloud_name = cloud_name
         self.build_pr = build_pr
@@ -193,7 +193,6 @@ class PrepareClusterBlock:
                     self.cluster_compute_name,
                     self.cluster_compute_dict,
                     self.cloud_name,
-                    self.cluster_compute_version,
                 )
             else:
                 compute_template_id = existing_terminated_cluster.compute_template_id
@@ -775,16 +774,15 @@ class PrepareClusterBlock:
         cluster_compute_name: Optional[str],
         cluster_compute_dict: Optional[CLUSTER_COMPUTE_DICT_TYPE],
         cloud_name: Optional[str],
-        cluster_compute_version: Optional[int] = None,
     ) -> str:
         """Returns the compute template ids of the cluster to be created.
         By default we return the default cluster compute, unless the user overrides.
         """
         # get cluster compute template
         if cluster_compute_name:
-            compute_template_id = self._get_cluster_compute_id_from_name(
-                cluster_compute_name, cluster_compute_version
-            )
+            compute_template_id = get_cluster_compute_from_name(
+                cluster_compute_name, self.api_client
+            ).id
         else:
             # If the user specifies _cluster_compute_dict use it, otherwise
             # use the default cluster compute template.
@@ -805,27 +803,6 @@ class PrepareClusterBlock:
             )
         return compute_template_id
 
-    def _get_cluster_compute_id_from_name(
-        self,
-        cluster_compute_name: str,
-        cluster_compute_version: Optional[int] = None,  # noqa: ARG002
-    ) -> str:
-        """Gets the cluster compute id given a cluster name."""
-        cluster_computes = self.api_client.search_compute_templates_api_v2_compute_templates_search_post(
-            ComputeTemplateQuery(
-                orgwide=True,
-                name={"equals": cluster_compute_name},
-                include_anonymous=True,
-                version=cluster_compute_version,
-            )
-        ).results
-        if len(cluster_computes) == 0:
-            raise ValueError(
-                f"The compute config {cluster_compute_name}"
-                " does not exist or you don't have sufficient permissions."
-            )
-        return cluster_computes[0].id
-
     def _register_compute_template(
         self, project_id: str, config_object: ComputeTemplateConfig
     ) -> str:
@@ -898,7 +875,6 @@ class PrepareClusterBlock:
                 self.cluster_compute_name,
                 self.cluster_compute_dict,
                 self.cloud_name,
-                self.cluster_compute_version,
             )
             if not self._is_equal_cluster_compute(
                 overriding_compute, running_cluster.compute_template_id
@@ -958,7 +934,6 @@ def create_prepare_cluster_block(  # noqa: PLR0913
     cluster_env_revision: Optional[int],
     ray: Any,
     log_output: bool = True,
-    cluster_compute_version: Optional[int] = None,
 ):
     """
     Function to get PrepareClusterBlock object. The PrepareClusterBlock object
@@ -982,7 +957,6 @@ def create_prepare_cluster_block(  # noqa: PLR0913
         cluster_env_revision=cluster_env_revision,
         ray=ray,
         log_output=log_output,
-        cluster_compute_version=cluster_compute_version,
     )
     prepare_cluster_block.prepare()
     return prepare_cluster_block
