@@ -380,7 +380,7 @@ class CloudFunctionalVerificationController(BaseController):
         cloud_provider: CloudProviders,
         *,
         version: str = "v1",
-        service_name: str = None,
+        service_name: Optional[str] = None,
         canary_percent: Optional[int] = None,
     ):
         """
@@ -719,10 +719,8 @@ class CloudFunctionalVerificationController(BaseController):
 
             # fail fast if any of the following logs are found
             return not any(
-                [
-                    failure_log in workspace_log.lines
-                    for failure_log in WORKSPACE_FAIL_FAST_MATCHING_LOGS
-                ]
+                failure_log in workspace_log.lines
+                for failure_log in WORKSPACE_FAIL_FAST_MATCHING_LOGS
             )
         elif function_type == CloudFunctionalVerificationType.SERVICE:
             service_events = self.api_client.get_service_events_api_v2_services_v2_service_id_events_get(
