@@ -1,13 +1,10 @@
-from abc import ABC, abstractmethod
 import asyncio
-from datetime import datetime, timedelta
 import os
 import random
 import string
 import time
-from typing import Any, cast, Dict, List, Optional, Tuple, Union
+from typing import Any, cast, Dict, List, Optional
 
-import aiohttp
 import click
 import tabulate
 import yaml
@@ -23,11 +20,13 @@ from anyscale.client.openapi_client import (
 from anyscale.client.openapi_client.models.create_internal_production_job import (
     CreateInternalProductionJob,
 )
+from anyscale.client.openapi_client.models.create_job_queue_config import (
+    CreateJobQueueConfig,
+)
 from anyscale.client.openapi_client.models.decorated_production_job import (
     DecoratedProductionJob,
 )
 from anyscale.client.openapi_client.models.ha_job_states import HaJobStates
-from anyscale.client.openapi_client.models.job_queue_config import JobQueueConfig
 from anyscale.controllers.base_controller import BaseController
 from anyscale.models.job_model import JobConfig
 from anyscale.project import infer_project_id
@@ -73,122 +72,6 @@ class MiniJobRun(BaseModel):
     error: Optional[str]
 
 
-class LogProvider(ABC):
-    @abstractmethod
-    def __init__(self, dns_name: str, anyscale_token: str, query: str) -> None:
-        pass
-
-    @abstractmethod
-    async def query(
-        self, start_timestamp_ns: int, end_timestamp_ns: int
-    ) -> List[Tuple[int, str]]:
-        """Query logs for a time range, return list of (timestamp_ns, log_line)."""
-
-    @abstractmethod
-    def close(self) -> None:
-        """Close any connection this log provider uses"""
-
-    def __enter__(self, *args, **kwargs):  # noqa: B027
-        pass
-
-    def __exit__(self, *args, **kwargs):
-        self.close()
-
-
-class LokiLogProvider(LogProvider):
-    def __init__(
-        self,
-        dns_name: str,
-        token: str,
-        query: str,
-        limit: int,
-        tenant_id: str,
-        direction: str,
-    ) -> None:
-        # https://grafana.com/docs/loki/latest/api/#get-lokiapiv1query_range
-        self.url = f"https://{dns_name}/loki/api/v1/query_range"
-        self.params: Dict[str, Union[str, int]] = {
-            "query": query,
-            "limit": limit,
-            "start": 0,
-            "end": int(time.time() * 1e9),
-            "direction": direction,
-        }
-        self.tenant_id = tenant_id
-        self.token = token
-        self.headers = {
-            "X-Scope-OrgID": self.tenant_id,
-            "Authorization": f"Bearer {self.token}",
-        }
-        self.ingress_auth_max_retry = 10
-        self.ingress_auth_retry_wait_time_s = 0.1
-        # We are retrying 401 here because there could be a race condition
-        # between pinging Loki and the auth token getting persisted and
-        # accessible by the ingress. In that case the status code will be
-        # 401 and thus need to retry.
-        # Also retrying 500 for possible transient errors.
-        self.ingress_auth_retry_status = {401, 500}
-        self.loki_query_max_retry_time_s = 60
-
-    async def __aenter__(self, *args, **kwargs):
-        self.session = aiohttp.ClientSession()
-        self.ingress_auth_current_retry_count = 0
-        return self
-
-    async def close(self):
-        await self.session.close()
-
-    async def __aexit__(self, *args, **kwargs):
-        await self.close()
-
-    async def query(
-        self, start_timestamp_ns: int, end_timestamp_ns: int
-    ) -> List[Tuple[int, str]]:
-        if start_timestamp_ns == 0:
-            # 10^9 nanoseconds in a second
-            start_timestamp_ns = int(
-                (datetime.now() - timedelta(days=7)).timestamp() * (10 ** 9)
-            )
-        self.params["start"] = start_timestamp_ns
-        self.params["end"] = end_timestamp_ns
-
-        query_start_time = time.time()
-        lines = []
-        while (
-            self.ingress_auth_current_retry_count < self.ingress_auth_max_retry
-            and time.time() - query_start_time < self.loki_query_max_retry_time_s
-        ):
-            async with self.session.get(
-                self.url,
-                params=self.params,
-                headers=self.headers,
-                timeout=self.loki_query_max_retry_time_s,
-            ) as resp:
-                if resp.status in self.ingress_auth_retry_status:
-                    self.ingress_auth_current_retry_count += 1
-                    await asyncio.sleep(self.ingress_auth_retry_wait_time_s)
-                    if (
-                        self.ingress_auth_current_retry_count
-                        < self.ingress_auth_max_retry
-                    ):
-                        continue
-
-                resp.raise_for_status()
-                resp_data = await resp.json()
-                if resp_data["status"] != "success":
-                    raise click.ClickException("Querying Anyscale log endpoint failed.")
-                if resp_data["data"]["resultType"] != "streams":
-                    raise click.ClickException("Invalid metrics type.")
-
-                for log_chunk in resp_data["data"]["result"]:
-                    # TODO(simon): potentially annotate the log with the log_chunk["stream"] metadata
-                    for timestamp, line in log_chunk["values"]:
-                        lines.append((int(timestamp), line))
-                return lines
-
-        return lines
-
-
 class JobController(BaseController):
     def __init__(
         self,
@@ -291,7 +174,6 @@ class JobController(BaseController):
             cluster_compute_id=job_config.compute_config_id,
             cluster_compute=job_config.compute_config,
             cloud=job_config.cloud,
-            cluster_compute_version=job_config.compute_config_version,
         )
 
         job_config.runtime_env = override_runtime_env_config(
@@ -311,7 +193,7 @@ class JobController(BaseController):
         )
 
         job_queue_config = (
-            JobQueueConfig(**job_config.job_queue_config)
+            CreateJobQueueConfig(**job_config.job_queue_config)
             if job_config.job_queue_config
             else None
         )
@@ -632,7 +514,10 @@ class JobController(BaseController):
         return job
 
     def _print_logs_using_streaming_job_logs(
-        self, job_run_id: str, follow: bool = False
+        self,
+        job_run_id: str,
+        follow: bool = False,
+        no_cluster_journal_events: bool = False,
     ) -> None:
         """Retrieves logs from the streaming job logs folder in S3/GCS"""
         logs = ""
@@ -655,6 +540,7 @@ class JobController(BaseController):
                     remove_escape_chars=False,
                     next_page_token=next_page_token,
                     cluster_journal_events_start_line=self._cluster_journal_events_start_line,
+                    no_cluster_journal_events=no_cluster_journal_events,
                 )
             )
 
@@ -691,6 +577,7 @@ class JobController(BaseController):
                     remove_escape_chars=False,
                     next_page_token=next_page_token,
                     cluster_journal_events_start_line=cluster_journal_events_start_line,
+                    no_cluster_journal_events=no_cluster_journal_events,
                 )
             )
 
@@ -706,9 +593,12 @@ class JobController(BaseController):
         job_name: Optional[str] = None,
         should_follow: bool = False,
         all_attempts: bool = False,
+        no_cluster_journal_events: bool = False,
     ) -> None:
         """
         Fetches logs for a production job using the streaming log S3 bucket.
+        Params:
+        - no_cluster_journal_events: Controls whether we print out cluster journal logs. (Normally printed to stderr)
         """
         job = self._wait_for_a_job_run(job_id, job_name)
         last_job_run_id: str = job.last_job_run_id
@@ -722,7 +612,9 @@ class JobController(BaseController):
                 job_run_id, f"Job Run Id: {job_run_id}", auto_close=True,
             )
             self._print_logs_using_streaming_job_logs(
-                job_run_id, follow=should_follow and job_run_id == last_job_run_id
+                job_run_id,
+                follow=should_follow and job_run_id == last_job_run_id,
+                no_cluster_journal_events=no_cluster_journal_events,
             )
 
     def _generate_random_job_name(self) -> str:
