@@ -129,7 +129,7 @@ class WorkspaceController(BaseController):
             + [f"bash -i -c {command}"]
         )
 
-        subprocess.run(ssh_command)
+        subprocess.run(ssh_command, check=False)
 
     def run_rsync(
         self,
@@ -199,4 +199,4 @@ class WorkspaceController(BaseController):
                 f"{ray_user_on_host}:{remote_path}",
             ]
 
-        subprocess.run(rsync_command)
+        subprocess.run(rsync_command, check=False)
