@@ -1,6 +1,6 @@
 from typing import Optional
 
-from pydantic import BaseModel
+from anyscale.anyscale_pydantic import BaseModel
 
 
 class FineTuneConfig(BaseModel):
@@ -9,3 +9,5 @@ class FineTuneConfig(BaseModel):
     valid_file: Optional[str]
     cloud_id: str
     suffix: Optional[str]
+    version: Optional[str]
+    instance_type: Optional[str]
