@@ -1,8 +1,7 @@
 import enum
 from typing import Any, Dict, Optional
 
-from pydantic import Field, root_validator
-
+from anyscale.anyscale_pydantic import Field, root_validator
 from anyscale.models.job_model import (
     _validate_and_modify_runtime_env,
     BaseHAJobConfig,
@@ -78,6 +77,16 @@ class ServiceConfig(BaseHAJobConfig):
         None, description="Target Service's configuration",
     )
 
+    auto_complete_rollout: Optional[bool] = Field(
+        None,
+        description="Flag to indicate whether or not to complete the rollout after the canary version reaches 100%.",
+    )
+
+    max_surge_percent: Optional[int] = Field(
+        None,
+        description="Max amount of excess capacity allocated during the rollout (0-100).",
+    )
+
     @root_validator
     def validates_config(cls, values) -> Dict[str, Any]:
         assert (
