@@ -441,7 +441,6 @@ def infer_project_id(  # noqa PLR0913
     cluster_compute_id: Optional[str] = None,
     cluster_compute: Optional[Union[str, Dict[str, Any]]] = None,
     cloud: Optional[str] = None,
-    cluster_compute_version: Optional[int] = None,
 ) -> str:
     if not project_id:
         # Check directory of .anyscale.yaml to decide whether to use default project.
@@ -453,7 +452,7 @@ def infer_project_id(  # noqa PLR0913
             # TODO: add cloud compute config args here
             if not cluster_compute_id and isinstance(cluster_compute, str):
                 cluster_compute_id = get_cluster_compute_from_name(
-                    cluster_compute, api_client, cluster_compute_version
+                    cluster_compute, api_client
                 ).id
             cluster_compute_config = (
                 ClusterComputeConfig(**cluster_compute)
