@@ -23,6 +23,7 @@ from anyscale.commands.list_commands import list_cli
 from anyscale.commands.login_commands import anyscale_login, anyscale_logout
 from anyscale.commands.logs_commands import log_cli
 from anyscale.commands.machine_commands import machine_cli
+from anyscale.commands.machine_pool_commands import machine_pool_cli
 from anyscale.commands.migrate_commands import migrate_cli
 from anyscale.commands.project_commands import anyscale_init, project_cli
 from anyscale.commands.schedule_commands import schedule_cli
@@ -128,6 +129,7 @@ cli.add_command(log_cli)
 cli.add_command(anyscale_login)
 cli.add_command(anyscale_logout)
 cli.add_command(machine_cli)
+cli.add_command(machine_pool_cli)
 
 ALIASES = {
     "h": anyscale_help,
