@@ -7,13 +7,6 @@ from time import sleep, time
 from typing import Any, Callable, Dict, List, Optional, Union
 
 from anyscale.api import configure_tcp_keepalive
-from anyscale.api_utils.job_logs_util import (
-    _get_logs_from_running_production_job,
-    _get_job_logs_from_storage_bucket,
-)
-from anyscale.api_utils.exceptions.log_retrieval_errors import (
-    ExpectedLogRetrievalError,
-)
 from anyscale.sdk.anyscale_client.models.cluster_compute_config import (
     ClusterComputeConfig,
 )
@@ -40,7 +33,7 @@ from anyscale.sdk.anyscale_client.models.cluster_environment_build import (
     ClusterEnvironmentBuild,
 )
 from anyscale.shared_anyscale_utils.headers import RequestHeaders
-from anyscale.cli_logger import BlockLogger, pad_string
+from anyscale.cli_logger import BlockLogger, StringLogger, pad_string
 from anyscale.cluster_compute import get_selected_cloud_id_or_default
 from anyscale.util import get_endpoint
 from anyscale.authenticate import AuthenticationBlock
@@ -364,7 +357,7 @@ class AnyscaleSDK(DefaultApi):  # type: ignore
         ).results
 
         if clusters:
-            
+
             sorted_clusters_list = sorted(clusters, key=lambda x: x.created_at, reverse=True)
             cluster: Cluster = sorted_clusters_list[0]
 
@@ -372,11 +365,11 @@ class AnyscaleSDK(DefaultApi):  # type: ignore
                 raise Exception(
                     f"Cluster {cluster_name} in project {project_id} is in state {cluster.state}. "
                     "Please terminate the cluster or use a new cluster name, then rerun launch_cluster."
-                )                
-            
+                )
+
             if not cluster_environment_build_id:
                 cluster_environment_build_id = cluster.cluster_environment_build_id
-            
+
             if not cluster_compute_id:
                 cluster_compute_id = cluster.cluster_compute_id
 
@@ -392,7 +385,7 @@ class AnyscaleSDK(DefaultApi):  # type: ignore
             )
 
             self.update_cluster(cluster.id, update_cluster_config)
-            
+
         else:
             if not cluster_environment_build_id:
                 # Use default cluster environment build when starting cluster without specifying an id
@@ -533,16 +526,12 @@ class AnyscaleSDK(DefaultApi):  # type: ignore
             return self.get_cluster(operation.cluster_id).result
 
     def fetch_production_job_logs(self, job_id: str) -> str:
+        from anyscale.controllers.job_controller import JobController
+
         """
         Retrieves logs for a job (eg. 'prodjob_123').
         - This will retrieve logs from only the latest job run of the job.
 
-        This will try in following order until it finds a retrieval method that returns the logs:
-        1. Ray Dashboard API on the cluster that the job is running on
-           NOTE: The caller of this function must be able to reach the Ray dashboard host.
-        2. (only for anyscale v1 clouds) Anyscale backend / cache layer which stores logs for recently finished jobs
-        3. The storage bucket for the Anyscale cloud the job (and its cluster) is running on
-
         Args:
             job_id (str): ID of a job
 
@@ -550,21 +539,11 @@ class AnyscaleSDK(DefaultApi):  # type: ignore
             str: All logs for the job
                 (so far, if the job is still running)
         """
-        job: ProductionJob = self.get_production_job(job_id).result
-        get_job_logs_fns: List[Callable[..., str]] = [
-            *(
-                [lambda: asyncio.run(_get_logs_from_running_production_job(self, job_id))] if job.state.current_state == HaJobStates.RUNNING else []
-            ),
-            lambda: asyncio.run(_get_job_logs_from_storage_bucket(self, job_id=job_id)),
-        ]
-        logs = ""
-        # Return logs from the first retrieval method that has non-empty logs
-        for fn in get_job_logs_fns:
-            with contextlib.suppress(ExpectedLogRetrievalError):
-                logs = fn()
-                if logs:
-                    return logs
-        return logs
+        string_logger = StringLogger()
+        job_controller = JobController(string_logger)
+        job_controller.logs(job_id, no_cluster_journal_events=True)
+
+        return string_logger.out_string
 
     def create_job(self, create_production_job, **kwargs):
         _upload_and_rewrite_working_dir_in_create_production_job(create_production_job)
@@ -600,7 +579,7 @@ class AnyscaleSDK(DefaultApi):  # type: ignore
             apply_production_service_v2_model.ray_serve_config = new_ray_serve_config
 
         return super().rollout_service_v2(apply_production_service_v2_model, **kwargs)
-    
+
     def rollout_service(self, apply_service_model, **kwargs):
         """
         If a local working_dir and upload_path are specified in ray_serve_config,
