@@ -1,23 +1,26 @@
 import copy
-from typing import Any, Dict, Optional
+from typing import Any, Dict, Optional, Union
 
-from anyscale._private.sdk.workload_sdk import WorkloadSDK
+from anyscale._private.models.image_uri import ImageURI
+from anyscale._private.workload import WorkloadSDK
 from anyscale.cli_logger import BlockLogger
 from anyscale.sdk.anyscale_client.models import (
     AccessConfig,
     ApplyServiceModel,
     ProductionServiceV2VersionModel,
-    RayGCSExternalStorageConfig,
+    RayGCSExternalStorageConfig as APIRayGCSExternalStorageConfig,
     ServiceConfig as ExternalAPIServiceConfig,
     ServiceEventCurrentState,
     ServiceModel,
 )
 from anyscale.service.models import (
+    RayGCSExternalStorageConfig,
     ServiceConfig,
     ServiceState,
     ServiceStatus,
     ServiceVersionStatus,
 )
+from anyscale.utils.runtime_env import parse_requirements_file
 from anyscale.utils.workspace_notification import (
     WorkspaceNotification,
     WorkspaceNotificationAction,
@@ -28,12 +31,15 @@ logger = BlockLogger()
 
 
 class ServiceSDK(WorkloadSDK):
+    SERVICE_VERSION_ID_TRUNCATED_LEN = 8
+
     def _override_application_runtime_envs(
         self,
         config: ServiceConfig,
         *,
         autopopulate_in_workspace: bool = True,
         cloud_id: Optional[str] = None,
+        workspace_requirements_path: Optional[str] = None,
     ) -> ServiceConfig:
         """Overrides the runtime_env of each application in the config.
 
@@ -56,11 +62,15 @@ class ServiceSDK(WorkloadSDK):
             excludes_override=config.excludes,
             cloud_id=cloud_id,
             autopopulate_in_workspace=autopopulate_in_workspace,
+            additional_py_modules=config.py_modules,
         )
         new_runtime_envs = self.override_and_load_requirements_files(
             new_runtime_envs,
             requirements_override=config.requirements,
-            autopopulate_in_workspace=autopopulate_in_workspace,
+            workspace_requirements_path=workspace_requirements_path,
+        )
+        new_runtime_envs = self.update_env_vars(
+            new_runtime_envs, env_vars_updates=config.env_vars
         )
 
         for app, new_runtime_env in zip(new_applications, new_runtime_envs):
@@ -124,36 +134,7 @@ class ServiceSDK(WorkloadSDK):
         )
         self.logger.info(f"curl {auth_token_header}{service.base_url}/")
 
-    def _build_apply_service_model_from_config(
-        self,
-        name: str,
-        config: ServiceConfig,
-        *,
-        in_place: bool = False,
-        canary_percent: Optional[int] = None,
-        max_surge_percent: Optional[int] = None,
-    ) -> ApplyServiceModel:
-        if config.containerfile is not None:
-            build_id = self.client.get_cluster_env_build_id_from_containerfile(
-                cluster_env_name=f"image-for-{name}",
-                containerfile=self.get_containerfile_contents(config.containerfile),
-            )
-        elif config.image_uri is not None:
-            build_id = self.client.get_cluster_env_build_id(image_uri=config.image_uri,)
-        else:
-            build_id = self.client.get_default_build_id()
-
-        compute_config_id = self.resolve_compute_config_to_id(config.compute_config)
-
-        # If a compute config was specified, we need to make sure to used the correct
-        # cloud_id when uploading local directories.
-        cloud_id = self.client.get_cloud_id(
-            compute_config_id=None
-            if config.compute_config is None
-            else compute_config_id
-        )
-        config = self._override_application_runtime_envs(config, cloud_id=cloud_id)
-
+    def _build_ray_serve_config(self, config: ServiceConfig) -> Dict[str, Any]:
         ray_serve_config: Dict[str, Any] = {"applications": config.applications}
         if config.http_options:
             ray_serve_config["http_options"] = config.http_options
@@ -162,21 +143,191 @@ class ServiceSDK(WorkloadSDK):
         if config.logging_config:
             ray_serve_config["logging_config"] = config.logging_config
 
-        ray_gcs_external_storage_config = None
-        if config.ray_gcs_external_storage_config is not None:
-            ray_gcs_external_storage_config = RayGCSExternalStorageConfig(
-                **config.ray_gcs_external_storage_config
+        return ray_serve_config
+
+    def _build_apply_service_model_for_in_place_update(  # noqa: PLR0912
+        self,
+        name: str,
+        config: ServiceConfig,
+        *,
+        canary_percent: Optional[int] = None,
+        max_surge_percent: Optional[int] = None,
+        existing_service: Optional[ServiceModel] = None,
+    ) -> ApplyServiceModel:
+        """Build the ApplyServiceModel for an in_place update.
+
+        in_place updates:
+            - must be performed on an existing service.
+            - cannot be performed while a rollout is active.
+            - do not support rollout options (canary_percent and max_surge_percent).
+            - do not support changing cluster-level config options (image, compute_config, etc.).
+
+        If cluster-level options are provided, a warning will be logged and they will be ignored.
+        """
+        if canary_percent is not None:
+            raise ValueError(
+                "canary_percent cannot be specified when doing an in_place update."
+            )
+        if max_surge_percent is not None:
+            raise ValueError(
+                "max_surge_percent cannot be specified when doing an in_place update."
+            )
+        if existing_service is None:
+            raise RuntimeError(
+                "in_place can only be used to update running services, "
+                f"but no service was found with name '{name}'."
+            )
+        if existing_service.current_state in {
+            ServiceEventCurrentState.TERMINATING,
+            ServiceEventCurrentState.TERMINATED,
+        }:
+            raise RuntimeError(
+                "in_place can only be used to update running services, "
+                f"but service '{name}' is terminated."
             )
 
+        if existing_service.canary_version is not None:
+            raise RuntimeError(
+                "in_place updates cannot be used while a rollout is in progress. "
+                "Complete the rollout or roll back first."
+            )
+
+        if any(
+            [
+                config.image_uri is not None,
+                config.containerfile is not None,
+                config.compute_config is not None,
+                config.ray_gcs_external_storage_config is not None,
+            ]
+        ):
+            self.logger.warning(
+                "Cluster-level options such as image and compute config "
+                "are ignored when performing an in_place update."
+            )
+
+        existing_config: ProductionServiceV2VersionModel = existing_service.primary_version
+        query_auth_token_enabled = existing_service.auth_token is not None
+        cloud_id = self.client.get_cloud_id(
+            compute_config_id=existing_config.compute_config_id
+        )
+        config = self._override_application_runtime_envs(
+            config,
+            cloud_id=cloud_id,
+            workspace_requirements_path=self.client.get_workspace_requirements_path(),
+        )
         return ApplyServiceModel(
             name=name,
-            project_id=self.client.get_project_id(),
-            ray_serve_config=ray_serve_config,
+            project_id=self.client.get_project_id(parent_cloud_id=cloud_id),
+            ray_serve_config=self._build_ray_serve_config(config),
+            build_id=existing_config.build_id,
+            compute_config_id=existing_config.compute_config_id,
+            rollout_strategy="IN_PLACE",
+            config=ExternalAPIServiceConfig(
+                access=AccessConfig(use_bearer_token=query_auth_token_enabled),
+            ),
+            ray_gcs_external_storage_config=existing_config.ray_gcs_external_storage_config,
+        )
+
+    def _build_apply_service_model_for_rollout(  # noqa: PLR0912
+        self,
+        name: str,
+        config: ServiceConfig,
+        *,
+        canary_percent: Optional[int] = None,
+        max_surge_percent: Optional[int] = None,
+        existing_service: Optional[ServiceModel] = None,
+    ) -> ApplyServiceModel:
+        """Build the ApplyServiceModel for a rolling update."""
+
+        build_id = None
+        if config.containerfile is not None:
+            build_id = self.client.get_cluster_env_build_id_from_containerfile(
+                cluster_env_name=f"image-for-service-{name}",
+                containerfile=self.get_containerfile_contents(config.containerfile),
+            )
+        elif config.image_uri is not None:
+            build_id = self.client.get_cluster_env_build_id_from_image_uri(
+                image_uri=ImageURI.from_str(config.image_uri)
+            )
+
+        if self.enable_image_build_for_tracked_requirements:
+            requirements_path_to_be_populated_in_runtime_env = None
+            requirements_path = self.client.get_workspace_requirements_path()
+            if requirements_path is not None:
+                requirements = parse_requirements_file(requirements_path)
+                if requirements:
+                    build_id = self.client.build_image_from_requirements(
+                        cluster_env_name=f"image-for-service-{name}",
+                        base_build_id=self.client.get_default_build_id(),
+                        requirements=requirements,
+                    )
+        else:
+            requirements_path_to_be_populated_in_runtime_env = (
+                self.client.get_workspace_requirements_path()
+            )
+
+        if build_id is None:
+            build_id = self.client.get_default_build_id()
+
+        compute_config_id, cloud_id = self.resolve_compute_config(config.compute_config)
+        if (
+            existing_service is not None
+            and existing_service.primary_version is not None
+        ):
+            existing_cloud_id = self.client.get_cloud_id(
+                compute_config_id=existing_service.primary_version.compute_config_id
+            )
+            if cloud_id != existing_cloud_id:
+                new_cloud = self.client.get_cloud(cloud_id=cloud_id)
+                assert new_cloud is not None
+                existing_cloud = self.client.get_cloud(cloud_id=existing_cloud_id)
+                assert existing_cloud is not None
+                raise ValueError(
+                    f"The cloud for a service cannot be changed once it's created. "
+                    f"Service '{name}' was created on cloud '{existing_cloud.name}', but "
+                    f"the provided config is for cloud '{new_cloud.name}'."
+                )
+
+        env_vars_from_workspace = self.client.get_workspace_env_vars()
+        if env_vars_from_workspace:
+            if config.env_vars:
+                # the precedence should be cli > workspace
+                env_vars_from_workspace.update(config.env_vars)
+                config = config.options(env_vars=env_vars_from_workspace)
+            else:
+                config = config.options(env_vars=env_vars_from_workspace)
+
+        config = self._override_application_runtime_envs(
+            config,
+            cloud_id=cloud_id,
+            workspace_requirements_path=requirements_path_to_be_populated_in_runtime_env,
+        )
+
+        ray_gcs_external_storage_config = None
+        if config.ray_gcs_external_storage_config is not None:
+            assert isinstance(
+                config.ray_gcs_external_storage_config, RayGCSExternalStorageConfig
+            )
+            ray_gcs_external_storage_config = APIRayGCSExternalStorageConfig(
+                enable=config.ray_gcs_external_storage_config.enabled,
+            )
+            if config.ray_gcs_external_storage_config.address is not None:
+                ray_gcs_external_storage_config.address = (
+                    config.ray_gcs_external_storage_config.address
+                )
+            if config.ray_gcs_external_storage_config.certificate_path is not None:
+                ray_gcs_external_storage_config.redis_certificate_path = (
+                    config.ray_gcs_external_storage_config.certificate_path
+                )
+        return ApplyServiceModel(
+            name=name,
+            project_id=self.client.get_project_id(parent_cloud_id=cloud_id),
+            ray_serve_config=self._build_ray_serve_config(config),
             build_id=build_id,
             compute_config_id=compute_config_id,
             canary_percent=canary_percent,
             max_surge_percent=max_surge_percent,
-            rollout_strategy="IN_PLACE" if in_place else "ROLLOUT",
+            rollout_strategy="ROLLOUT",
             config=ExternalAPIServiceConfig(
                 access=AccessConfig(use_bearer_token=config.query_auth_token_enabled),
             ),
@@ -203,12 +354,10 @@ class ServiceSDK(WorkloadSDK):
         if max_surge_percent is not None:
             if not isinstance(max_surge_percent, int):
                 raise TypeError("max_surge_percent must be an int.")
-
             if max_surge_percent < 0 or max_surge_percent > 100:
                 raise ValueError("max_surge_percent must be between 0 and 100.")
 
         name = config.name or self._get_default_name()
-
         existing_service: Optional[ServiceModel] = self.client.get_service(name)
         if existing_service is None:
             self.logger.info(f"Starting new service '{name}'.")
@@ -217,16 +366,24 @@ class ServiceSDK(WorkloadSDK):
         else:
             self.logger.info(f"Updating existing service '{name}'.")
 
-        service: ServiceModel = self.client.rollout_service(
-            self._build_apply_service_model_from_config(
+        if in_place:
+            model = self._build_apply_service_model_for_in_place_update(
                 name,
                 config,
-                in_place=in_place,
                 canary_percent=canary_percent,
                 max_surge_percent=max_surge_percent,
+                existing_service=existing_service,
+            )
+        else:
+            model = self._build_apply_service_model_for_rollout(
+                name,
+                config,
+                canary_percent=canary_percent,
+                max_surge_percent=max_surge_percent,
+                existing_service=existing_service,
             )
-        )
 
+        service: ServiceModel = self.client.rollout_service(model)
         self._log_deployed_service_info(service, canary_percent=canary_percent)
 
     def rollback(
@@ -239,9 +396,7 @@ class ServiceSDK(WorkloadSDK):
         if model is None:
             raise RuntimeError(f"Service with name '{name}' was not found.")
 
-        return self.client.rollback_service(
-            model.id, max_surge_percent=max_surge_percent,
-        )
+        self.client.rollback_service(model.id, max_surge_percent=max_surge_percent)
 
     def terminate(self, name: Optional[str] = None):
         if name is None:
@@ -251,21 +406,7 @@ class ServiceSDK(WorkloadSDK):
         if model is None:
             raise RuntimeError(f"Service with name '{name}' was not found.")
 
-        return self.client.terminate_service(model.id,)
-
-    def _strip_none_values_from_dict(self, d: Dict) -> Dict:
-        """Return a copy of the dictionary without any keys whose values are None.
-
-        Recursively calls into any dictionary values.
-        """
-        result = {}
-        for k, v in d.items():
-            if isinstance(v, dict):
-                result[k] = self._strip_none_values_from_dict(v)
-            elif v is not None:
-                result[k] = v
-
-        return result
+        self.client.terminate_service(model.id)
 
     def _service_version_model_to_status(
         self,
@@ -280,18 +421,24 @@ class ServiceSDK(WorkloadSDK):
 
         ray_gcs_external_storage_config = None
         if model.ray_gcs_external_storage_config is not None:
-            ray_gcs_external_storage_config = self._strip_none_values_from_dict(
-                model.ray_gcs_external_storage_config.to_dict()
+            ray_gcs_external_storage_config = RayGCSExternalStorageConfig(
+                enabled=model.ray_gcs_external_storage_config.enable,
+                address=model.ray_gcs_external_storage_config.address,
+                certificate_path=model.ray_gcs_external_storage_config.redis_certificate_path,
             )
 
         return ServiceVersionStatus(
-            name=model.version,
+            # NOTE(edoakes): the "version ID" exposed in the UI and tagged in the metrics is
+            # not actually the DB ID, but a truncated version of it. We should store this
+            # in the DB to avoid breakages, but for now I'm copying the existing logic.
+            id=model.id[-self.SERVICE_VERSION_ID_TRUNCATED_LEN :],
+            state=model.current_state,
             # NOTE(edoakes): there is also a "current_weight" field but it does not match the UI.
             weight=model.weight,
             config=ServiceConfig(
                 name=service_name,
                 applications=model.ray_serve_config["applications"],
-                image_uri=image_uri,
+                image_uri=str(image_uri),
                 compute_config=self.get_user_facing_compute_config(
                     model.compute_config_id
                 ),
@@ -331,10 +478,10 @@ class ServiceSDK(WorkloadSDK):
 
         # If we add a new state to the backend, old clients may not recognize it.
         # Rather than erroring out and causing old code to crash, return UNKNOWN.
+        state: Union[str, ServiceState] = ServiceState.UNKNOWN
         try:
             state = ServiceState(model.current_state)
         except ValueError:
-            state = ServiceState.UNKNOWN
             self.logger.warning(
                 f"Got unrecognized state: '{model.current_state}'. "
                 "You likely need to update the 'anyscale' package. "
