@@ -833,8 +833,9 @@ def prepare_cloudformation_template(
             f'{{"subnet_id": !Ref Subnet{i}, "availability_zone": !GetAtt Subnet{i}.AvailabilityZone}}'
         )
 
-        efs_mount_targets.append(
-            f"""
+        if not is_anyscale_hosted:
+            efs_mount_targets.append(
+                f"""
   EFSMountTarget{i}:
     Type: AWS::EFS::MountTarget
     Properties:
@@ -842,7 +843,7 @@ def prepare_cloudformation_template(
         SecurityGroups:
           - !Ref AnyscaleSecurityGroup
         SubnetId: !Ref Subnet{i}"""
-        )
+            )
 
     body = body.replace("$SUBNETS_TEMPLATES", "\n".join(subnet_templates))
     body = body.replace(
@@ -852,7 +853,8 @@ def prepare_cloudformation_template(
         "$SUBNETS_WITH_AVAILABILITY_ZONES",
         f'[{",".join(subnets_with_availability_zones)}]',
     )
-    body = body.replace("$EFSMountTargets", "\n".join(efs_mount_targets))
+    if not is_anyscale_hosted:
+        body = body.replace("$EFSMountTargets", "\n".join(efs_mount_targets))
 
     iam_policy_parameters = [
         generate_inline_policy_parameter(policy) for policy in ANYSCALE_IAM_POLICIES
@@ -895,7 +897,7 @@ def get_latest_ray_version():
         latest_version = response.json()["info"]["version"]
     except Exception as e:  # noqa: BLE001
         log.debug(
-            f"Unable to get latest Ray version from https://pypi.org/pypi/ray/json {str(e)}"
+            f"Unable to get latest Ray version from https://pypi.org/pypi/ray/json {e!s}"
         )
         latest_version = DEFAULT_RAY_VERSION
     return latest_version
@@ -940,9 +942,7 @@ def validate_job_config_dict(
         config_dict["compute_config"], str
     ):
         compute_config = get_cluster_compute_from_name(
-            config_dict["compute_config"],
-            api_client,
-            config_dict.get("compute_config_version"),
+            config_dict["compute_config"], api_client
         )
     elif "compute_config_id" in config_dict:
         cluster_compute_id = config_dict["compute_config_id"]
@@ -1023,7 +1023,8 @@ def populate_unspecified_cluster_configs_from_current_workspace(
 
 
 def filter_actions_from_policy_document(
-    policy_document: Dict[Any, Any], action_filter: Callable[[Dict], bool] = None
+    policy_document: Dict[Any, Any],
+    action_filter: Optional[Callable[[Dict], bool]] = None,
 ) -> set:
     if action_filter is None:
         action_filter = lambda statement: statement["Effect"] == "Allow"  # noqa: E731
@@ -1038,7 +1039,7 @@ def filter_actions_from_policy_document(
 def filter_actions_associated_with_role(
     boto3_session: boto3.Session,
     role: Boto3Resource,
-    action_filter: Callable[[Dict], bool] = None,
+    action_filter: Optional[Callable[[Dict], bool]] = None,
 ) -> Set[str]:
     iam = boto3_session.resource("iam")
     attached_policy_documents = [
