@@ -90,7 +90,7 @@ class PropertyDifference(BaseModel):
             )
             is not None
         ):
-            if self.DifferenceType == "ADD" or self.DifferenceType == "REMOVE":
+            if self.DifferenceType in ("ADD", "REMOVE"):
                 return True
             else:
                 raise ClickException(f"Drift {self.json()} cannot be resolved.")
@@ -255,7 +255,7 @@ class AWSCloudformationHandler:
         with self.logger.spinner("Creating change set for cloud update..."):
             response = self.cfn_client.create_change_set(
                 StackName=stack_name,
-                ChangeSetName=f"AnyscaleCloudUpdate{str(secrets.token_hex(4))}",
+                ChangeSetName=f"AnyscaleCloudUpdate{secrets.token_hex(4)!s}",
                 TemplateBody=updated_template_body,
                 Parameters=updated_parameters,
                 Capabilities=["CAPABILITY_NAMED_IAM", "CAPABILITY_AUTO_EXPAND"],
