@@ -1,11 +1,14 @@
 from dataclasses import dataclass
 import inspect
-from typing import Any, Callable, Dict, Generic, List, Optional, TypeVar
+from typing import Any, Callable, Dict, Generic, List, Optional, TYPE_CHECKING, TypeVar
 
 from anyscale.anyscale_pydantic import BaseModel
 from anyscale.client.openapi_client.models.session import Session
 from anyscale.sdk.anyscale_client.models.cluster import Cluster
-from anyscale.sdk.anyscale_client.sdk import AnyscaleSDK
+
+
+if TYPE_CHECKING:
+    from anyscale.sdk.anyscale_client.sdk import AnyscaleSDK
 
 
 try:
@@ -45,7 +48,7 @@ def _multiclient_supported() -> bool:
     return "_context_to_restore" in _context_params
 
 
-def find_project_id(sdk: AnyscaleSDK, project_name: str) -> Optional[str]:
+def find_project_id(sdk: "AnyscaleSDK", project_name: str) -> Optional[str]:
     """Return id if a project of a given name exists."""
     resp = sdk.search_projects({"name": {"equals": project_name}})
     if len(resp.results) > 0:
@@ -55,7 +58,7 @@ def find_project_id(sdk: AnyscaleSDK, project_name: str) -> Optional[str]:
 
 
 def get_cluster(
-    sdk: AnyscaleSDK, project_id: str, session_name: str
+    sdk: "AnyscaleSDK", project_id: str, session_name: str
 ) -> Optional[Session]:
     """Query Anyscale for the given cluster's metadata."""
     clusters = sdk.search_clusters(
