@@ -1,3 +1,4 @@
+import copy
 import os
 from string import Template
 import time
@@ -83,18 +84,23 @@ def append_project_iam_policy(
 
 
 def enable_project_apis(
-    factory: GoogleCloudClientFactory, project_id: str, logger: CloudSetupLogger
+    factory: GoogleCloudClientFactory,
+    project_id: str,
+    logger: CloudSetupLogger,
+    enable_head_node_fault_tolerance: bool = False,
 ):
     """ Automatically enable APIs that Anyscale needs.
     """
     try:
         service_usage_client = factory.build("serviceusage", "v1")
+        api_list = copy.deepcopy(GCP_REQUIRED_APIS)
+        if enable_head_node_fault_tolerance:
+            api_list.append("redis.googleapis.com")
 
         response = (
             service_usage_client.services()
             .batchEnable(
-                parent="projects/" + project_id,
-                body={"serviceIds": GCP_REQUIRED_APIS,},
+                parent="projects/" + project_id, body={"serviceIds": api_list},
             )
             .execute()
         )
@@ -300,16 +306,24 @@ def generate_deployment_manager_config(  # noqa: PLR0913
     anyscale_aws_account: str,
     organization_id: str,
     enable_head_node_fault_tolerance: bool,
+    *,
+    vpc_name: Optional[str] = None,
     subnet_cidr: str = "10.0.0.0/20",
     zone: str = "b",  # some regions like us-east1 or europe-west1 don't have zone "a"
     fileshare_capacity_gb: int = 1024,
+    control_plane_service_account_email: Optional[str] = None,
 ) -> str:
     """
     Generate the deployment manager config for Anyscale cloud setup.
     """
-    with open(
+    file_path = (
         os.path.join(anyscale.conf.ROOT_DIR_PATH, "anyscale-cloud-setup-gcp.yaml")
-    ) as f:
+        if not vpc_name
+        else os.path.join(
+            anyscale.conf.ROOT_DIR_PATH, "anyscale-cloud-setup-gcp-oa.yaml"
+        )
+    )
+    with open(file_path) as f:
         body = f.read()
 
     if enable_head_node_fault_tolerance:
@@ -332,6 +346,8 @@ def generate_deployment_manager_config(  # noqa: PLR0913
         "SUBNET_CIDR": subnet_cidr,
         "ZONE": zone,
         "FILESHARE_CAPACITY_GB": fileshare_capacity_gb,
+        "VPC_NAME": vpc_name,
+        "ANYSCALE_CONTROL_PLANE_SERVICE_ACCOUNT_EMAIL": control_plane_service_account_email,
     }
 
     return deployment_manager_config.substitute(params)
