@@ -1,4 +1,4 @@
-from typing import List
+from typing import List, Optional
 
 from anyscale.client.openapi_client.models import LogFileChunk
 from anyscale.client.openapi_client.models.node_type import NodeType
@@ -23,19 +23,26 @@ class LogGroupFile:
 
     def get_target_path(self) -> str:
         assert len(self.chunks) >= 1
-        # Rewrite into:
-        # logs/<cluster-id>/<session-id>/<node-type>-<node-ip>-<instance-id>/<file_name>.log
         sample_chunk = self.chunks[0]
         node_type = "head" if sample_chunk.node_type == NodeType.HEAD_NODE else "worker"
-        return "/".join(
-            [
-                "logs",
-                sample_chunk.cluster_id,
-                sample_chunk.session_id,
-                f"{node_type}-{sample_chunk.node_ip}-{sample_chunk.instance_id}",
-                sample_chunk.file_name,
-            ]
-        )
+
+        # We have special-case handling here to deal with non-standard log files that are sourced from something that's
+        # a single node (e.g. webterminal logs + job driver logs do not have node information associated with them):
+
+        path = "/".join(["logs", sample_chunk.cluster_id, sample_chunk.session_id])
+
+        # If the logs came from a specific node, add node information.
+        if sample_chunk.node_ip and sample_chunk.instance_id:
+            path = (
+                path
+                + "/"
+                + f"{node_type}-{sample_chunk.node_ip}-{sample_chunk.instance_id}"
+            )
+
+        # Add the rest of the destination filename (this might be something like "dashboard.log" or "serve/health.log").
+        path = path + "/" + sample_chunk.file_name
+
+        return path
 
     def get_size(self) -> int:
         return sum([chunk.size for chunk in self.chunks])
@@ -105,8 +112,9 @@ class LogGroupSession:
 
 
 class LogGroup:
-    def __init__(self):
+    def __init__(self, bearer_token: Optional[str] = None):
         self.sessions: List[LogGroupSession] = []
+        self.bearer_token = bearer_token
 
     def insert_chunk(self, chunk: LogFileChunk):
         for session in self.sessions:
