@@ -6,7 +6,16 @@ import logging
 import os
 from pathlib import Path
 import tempfile
-from typing import Any, Callable, Dict, Generator, List, Optional, Tuple, TYPE_CHECKING
+from typing import (
+    Any,
+    Callable,
+    Dict,
+    Generator,
+    List,
+    Optional,
+    Tuple,
+    TYPE_CHECKING,
+)
 from urllib.parse import urlparse
 
 import click
@@ -33,7 +42,7 @@ if TYPE_CHECKING:
 
 logger = logging.getLogger(__name__)
 
-
+# TODO(austin): refactor to read requirements.txt and .skip_packages_tracking from s3 or gcs directly.
 # Default cluster storage directory.
 CLUSTER_STORAGE_DIR = "/mnt/cluster_storage"
 
@@ -87,7 +96,7 @@ def _upload_file_to_google_cloud_storage(file: str, bucket: str, object_name: st
         blob.upload_from_filename(file)
     except Exception as e:  # noqa: BLE001
         raise click.ClickException(
-            f"Failed to upload the working directory to Google Cloud Storage. Error {repr(e)}"
+            f"Failed to upload the working directory to Google Cloud Storage. Error {e!r}"
             "Please validate you have exported cloud credentials with the correct write permissions and the intended bucket exists in your Cloud Storage account. "
             "If you do not desire to upload your working directory, please set your working directory to a public remote URI or remove the runtime_environment from you service yaml."
         ) from e
@@ -108,7 +117,7 @@ def _upload_file_to_s3(file: str, bucket: str, object_key: str):
         s3_client.upload_file(file, bucket, object_key)
     except Exception as e:  # noqa: BLE001
         raise click.ClickException(
-            f"Failed to upload the working directory to S3. Error {repr(e)}"
+            f"Failed to upload the working directory to S3. Error {e!r}"
             "Please validate you have exported cloud credentials with the correct write permissions and the intended bucket exists in your S3 account. "
             "If you do not desire to upload your working directory, please set your working directory to a public remote URI or remove the runtime_environment from you service yaml."
         ) from e
@@ -188,7 +197,7 @@ def upload_and_rewrite_working_dir(
     After uploading, deletes the "upload_path" field because it is no longer used and is not a valid
     OSS runtime env field.
     """
-    if runtime_env_json.get("working_dir", None) is None:
+    if runtime_env_json.get("working_dir") is None:
         return runtime_env_json
 
     working_dir = runtime_env_json["working_dir"]
@@ -197,7 +206,7 @@ def upload_and_rewrite_working_dir(
         return runtime_env_json
 
     upload_path = runtime_env_json["upload_path"]
-    excludes = runtime_env_json.get("excludes", None)
+    excludes = runtime_env_json.get("excludes")
     with zip_local_dir(working_dir, excludes=excludes) as (zip_file_path, _, hash_val):
         uploaded_zip_file_name = f"_anyscale_pkg_{hash_val}.zip"
         final_uploaded_filepath = upload_file_to_remote_storage_fn(
@@ -247,45 +256,64 @@ def override_runtime_env_config(
             # If upload_path is specified
             # we back up the current working dir to the specified path
             new_runtime_env = upload_and_rewrite_working_dir(existing_runtime_env)
+        elif is_anyscale_workspace() and "ANYSCALE_SESSION_ID" in os.environ:
+            # If submitting job v2 from workspaces and no upload_path is specified,
+            # we back up the current workspace content into S3
+            cluster_id = os.environ["ANYSCALE_SESSION_ID"]
+
+            decorated_cluster = api_client.get_decorated_cluster_api_v2_decorated_sessions_cluster_id_get(
+                cluster_id
+            ).result
+            cloud_id = decorated_cluster.cloud.id
+
+            workspace_id = os.environ["ANYSCALE_EXPERIMENTAL_WORKSPACE_ID"]
+
+            new_runtime_env = infer_upload_path_and_rewrite_working_dir(
+                api_client=api_client,
+                existing_runtime_env=existing_runtime_env,
+                workload_type=workload_type,
+                cloud_id=cloud_id,
+                log=log,
+                workspace_id=workspace_id,
+            )
         else:
-            if is_anyscale_workspace() and "ANYSCALE_SESSION_ID" in os.environ:
-                # If submitting job v2 from workspaces and no upload_path is specified,
-                # we back up the current workspace content into S3
-                cluster_id = os.environ["ANYSCALE_SESSION_ID"]
-
-                decorated_cluster = api_client.get_decorated_cluster_api_v2_decorated_sessions_cluster_id_get(
-                    cluster_id
-                ).result
-                cloud_id = decorated_cluster.cloud.id
-
-                workspace_id = os.environ["ANYSCALE_EXPERIMENTAL_WORKSPACE_ID"]
-
-                new_runtime_env = infer_upload_path_and_rewrite_working_dir(
-                    api_client=api_client,
-                    existing_runtime_env=existing_runtime_env,
-                    workload_type=workload_type,
-                    cloud_id=cloud_id,
-                    log=log,
-                    workspace_id=workspace_id,
-                )
-            else:
-                compute_template: ComputeTemplate = anyscale_api_client.get_compute_template(
-                    compute_config_id
-                ).result
-                cloud_id = compute_template.config.cloud_id
-                new_runtime_env = infer_upload_path_and_rewrite_working_dir(
-                    api_client=api_client,
-                    existing_runtime_env=existing_runtime_env,
-                    workload_type=workload_type,
-                    cloud_id=cloud_id,
-                    log=log,
-                )
+            compute_template: ComputeTemplate = anyscale_api_client.get_compute_template(
+                compute_config_id
+            ).result
+            cloud_id = compute_template.config.cloud_id
+            new_runtime_env = infer_upload_path_and_rewrite_working_dir(
+                api_client=api_client,
+                existing_runtime_env=existing_runtime_env,
+                workload_type=workload_type,
+                cloud_id=cloud_id,
+                log=log,
+            )
 
         return new_runtime_env
     else:
         return existing_runtime_env
 
 
+def parse_dot_env_file(dot_env_bytes: bytes) -> Dict[str, str]:
+    """Parse a .env file and return a dictionary of key-value pairs."""
+    dot_env = dot_env_bytes.split(b"\x00")
+    ret = {}
+    for kv in dot_env:
+        if len(kv) == 0:
+            # skip empty lines
+            continue
+        segs = kv.split(b"=", 1)
+        if len(segs) == 2:
+            key, value = segs
+            try:
+                ret[key.decode().strip()] = value.decode().strip()
+            except UnicodeDecodeError:
+                logger.error(f"Failed to decode env var entry: {kv!r}")
+        else:
+            logger.error(f"Invalid env var entry: {kv!r}")
+    return ret
+
+
 def parse_requirements_file(path: str) -> Optional[List[str]]:
     """TODO: add comment."""
     requirements_file = Path(path)
