@@ -1,3 +1,4 @@
+from collections import deque
 import json
 from typing import Any, cast, Dict, Optional, Tuple
 
@@ -15,6 +16,7 @@ from anyscale.webterminal.utils import (
 
 
 BASH_CMD = f"source {BASH_PREEXEC_CONFIG} && clear\n"
+TERMINAL_READ_BUFFER_MAXLEN = 1000
 
 
 class TermState:
@@ -59,6 +61,14 @@ class AnyscaleTermManager(NamedTermManager):  # type: ignore
         self.is_exec_command[name] = None
         self.terminal_state[name] = TermState()
         term.ptyproc.write(BASH_CMD)
+
+        # Overwrite the read buffer's maxlen to TERMINAL_READ_BUFFER_MAXLEN, so terminal history can be longer.
+        # terminado==0.12.1 set read_buffer maxlen to 10, but in newer version, it's set to 1000.
+        # We should consider to upgrade terminado in the future and remove the overwriting.
+        # The source that control the maxlen is here:
+        # https://github.com/jupyter/terminado/blob/f586aca16a70bbb0d36bc5475345022d2b9ca587/terminado/management.py#LL48C14-L48C25
+        term.read_buffer = deque([], maxlen=TERMINAL_READ_BUFFER_MAXLEN)
+
         return name, term
 
     def terminal_count(self, include_all: bool = False) -> int:
@@ -218,13 +228,13 @@ class ExecTerminalHandler(tornado.web.RequestHandler):
 
     async def post(self) -> None:
         """
-          POST /exec/ accepts { shell_command, command_id }
-            - shell_command, the command to run
-            - command_id, the id of the session_command entry created on the backend
+        POST /exec/ accepts { shell_command, command_id }
+          - shell_command, the command to run
+          - command_id, the id of the session_command entry created on the backend
 
-            returns: { web_terminal_id }
-            this is the web_terminal in which this command was run in.
-            We can use this to kill the command.
+          returns: { web_terminal_id }
+          this is the web_terminal in which this command was run in.
+          We can use this to kill the command.
         """
         # if default is not specified, the argument is considered required and an exeception will
         # be raised by tornado.
