@@ -255,13 +255,17 @@ def test_rollout_service(create_service_succeed: bool, canary_percent: Optional[
                 description=f"service for cloud {mock_cloud_id} functional verification",
                 project_id=mock_project_id,
                 ray_serve_config={
-                    "import_path": "serve_hello:entrypoint",
-                    "runtime_env": {
-                        "working_dir": "https://github.com/anyscale/docs_examples/archive/refs/heads/main.zip",
-                        "env_vars": {
-                            "SERVE_RESPONSE_MESSAGE": f"cloud functional verification {mock_version}",
-                        },
-                    },
+                    "applications": [
+                        {
+                            "import_path": "serve_hello:entrypoint",
+                            "runtime_env": {
+                                "working_dir": "https://github.com/anyscale/docs_examples/archive/refs/heads/main.zip",
+                                "env_vars": {
+                                    "SERVE_RESPONSE_MESSAGE": f"cloud functional verification {mock_version}",
+                                },
+                            },
+                        }
+                    ],
                 },
                 version=mock_version,
                 build_id=mock_cluster_env_build_id,
@@ -345,15 +349,16 @@ def test_verify_service(
 
 
 @pytest.mark.parametrize(
-    ("get_current_status_error", "status_not_allowed", "expected_result"),
+    ("get_current_status_error", "status_not_allowed", "fail_fast", "expected_result"),
     [
-        pytest.param(False, False, True, id="happy-path"),
-        pytest.param(True, False, False, id="get-current-status-error"),
-        pytest.param(False, True, False, id="status-not-allowed"),
+        pytest.param(False, False, False, True, id="happy-path"),
+        pytest.param(True, False, False, False, id="get-current-status-error"),
+        pytest.param(False, True, False, False, id="status-not-allowed"),
+        pytest.param(False, False, True, False, id="fail-fast"),
     ],
 )
 def test_poll_until_active(
-    get_current_status_error, status_not_allowed, expected_result
+    get_current_status_error, status_not_allowed, fail_fast, expected_result
 ):
     mock_function_type = Mock()
     mock_function_id = "mock_function_id"
@@ -365,10 +370,11 @@ def test_poll_until_active(
     )
     if get_current_status_error:
         mock_get_current_status.side_effect = ClickException("mock error")
+    mock_render_event_log = Mock(return_value=not fail_fast)
     with patch.multiple(
         "anyscale.controllers.cloud_functional_verification_controller.CloudFunctionalVerificationController",
         _update_task_in_step_progress=Mock(),
-        _render_event_log=Mock(),
+        _render_event_log=mock_render_event_log,
     ), patch.multiple(
         "anyscale.controllers.cloud_functional_verification_controller",
         POLL_INTERVAL_SECONDS=0,
@@ -403,6 +409,81 @@ def test_poll_until_active(
                     assert e.match("Failed to get")
                 if status_not_allowed:
                     assert e.match("is in an unexpected state:")
+                if fail_fast:
+                    assert e.match(
+                        "verification failed! Please check the logs and terminate"
+                    )
+
+
+@pytest.mark.parametrize(
+    ("no_new_logs", "fail_fast"),
+    [
+        pytest.param(True, False, id="no-new-logs"),
+        pytest.param(False, False, id="new-logs"),
+        pytest.param(False, True, id="failed"),
+    ],
+)
+def test_render_event_log_workspace(
+    no_new_logs: bool, fail_fast: bool,
+):
+    controller = CloudFunctionalVerificationController(Mock())
+    function = Mock(cluster_id="mock_cluster_id")
+    mock_log_num = 10
+    mock_new_log_num = 10 if no_new_logs else 14
+    controller.event_log_num[CloudFunctionalVerificationType.WORKSPACE] = mock_log_num
+    mock_add_row = Mock()
+    controller.event_log_tables[CloudFunctionalVerificationType.WORKSPACE] = Mock(
+        add_row=mock_add_row
+    )
+    mock_get_logs = Mock(
+        return_value=Mock(
+            result=Mock(
+                lines="new-logs" if not fail_fast else "Failed to execute",
+                num_lines=mock_new_log_num,
+            )
+        )
+    )
+    controller.api_client = Mock(
+        get_startup_logs_api_v2_sessions_session_id_startup_logs_get=mock_get_logs
+    )
+    assert controller._render_event_log(
+        CloudFunctionalVerificationType.WORKSPACE, function,
+    ) == (not fail_fast)
+    assert (
+        controller.event_log_num[CloudFunctionalVerificationType.WORKSPACE]
+        == mock_new_log_num
+    )
+    if no_new_logs:
+        mock_add_row.assert_not_called()
+    else:
+        mock_add_row.assert_called_once()
+
+
+@pytest.mark.parametrize("no_new_logs", [True, False])
+def test_render_event_log_service(no_new_logs: bool):
+    controller = CloudFunctionalVerificationController(Mock())
+    controller._format_service_event_log = Mock()  # type: ignore
+    function = Mock(id="mock_cluster_id")
+    mock_log_num = 10
+    mock_new_log_num = 10 if no_new_logs else 14
+    controller.event_log_num[CloudFunctionalVerificationType.SERVICE] = mock_log_num
+    mock_add_row = Mock()
+    controller.event_log_tables[CloudFunctionalVerificationType.SERVICE] = Mock(
+        add_row=mock_add_row
+    )
+    mock_get_logs = Mock(return_value=Mock(results=["log"] * mock_new_log_num))
+    controller.api_client = Mock(
+        get_service_events_api_v2_services_v2_service_id_events_get=mock_get_logs
+    )
+    assert (
+        controller._render_event_log(CloudFunctionalVerificationType.SERVICE, function)
+        is True
+    )
+    assert (
+        controller.event_log_num[CloudFunctionalVerificationType.SERVICE]
+        == mock_new_log_num
+    )
+    assert mock_add_row.call_count == mock_new_log_num - mock_log_num
 
 
 def test_poll_until_active_timeout():
