@@ -6,7 +6,7 @@ from anyscale.client.openapi_client.models import (
     MachineConnectionState,
     MachineInfo,
 )
-from frontend.cli.anyscale.controllers.machine_controller import MachineController
+from anyscale.controllers.machine_controller import MachineController
 
 
 def test_list_machines(mock_auth_api_client: str) -> None:
