@@ -19,6 +19,7 @@ from anyscale.utils.runtime_env import (
     autopopulate_runtime_env_for_workspace,
     infer_upload_path_and_rewrite_working_dir,
     override_runtime_env_config,
+    parse_dot_env_file,
 )
 from anyscale.utils.workload_types import Workload
 
@@ -547,3 +548,21 @@ def test_get_gs_bucket_gcp(
     assert (
         _get_cloud_gs_bucket_from_cloud(mock_api_client, cloud) == expected_bucket_name
     )
+
+
+@pytest.mark.parametrize(
+    ("raw", "expected", "error_in_logs"),
+    [
+        (b"", {}, False),
+        (b"\xff=abc\x00a=b", {"a": "b"}, False),  # invalid utf-8 should be ignored
+        (b"a=b\x00b=c", {"a": "b", "b": "c"}, False),
+        (b"a=b\x00b='c\nd'", {"a": "b", "b": "'c\nd'"}, False),
+        (b"a=b\x00b=c=d", {"a": "b", "b": "c=d"}, False),
+        (b"a=b\x00b=c=d\x00a\x00k=d\n", {"a": "b", "b": "c=d", "k": "d"}, True),
+        (b"  a=b\x00b=c=d   \x00a\x00k=d   \n", {"a": "b", "b": "c=d", "k": "d"}, True),
+    ],
+)
+def test_parse_dot_env_file(raw, expected, error_in_logs, caplog):
+    assert parse_dot_env_file(raw) == expected
+    if not error_in_logs:
+        assert "Invalid env var entry:" not in caplog.text
