@@ -1,4 +1,3 @@
-import copy
 import json
 import os
 import re
@@ -30,10 +29,11 @@ from anyscale.util import (
     DEFAULT_RAY_VERSION,
     extract_versions_from_image_name,
     filter_actions_from_policy_document,
+    get_cluster_model_for_current_workspace,
     get_latest_ray_version,
     get_ray_and_py_version_for_default_cluster_env,
     poll,
-    populate_dict_with_workspace_config_if_exists,
+    populate_unspecified_cluster_configs,
     prepare_cloudformation_template,
     sleep_till,
     str_data_size,
@@ -42,6 +42,7 @@ from anyscale.util import (
 )
 from anyscale.utils.cloud_utils import CloudSetupError
 from anyscale.utils.name_utils import gen_valid_name
+from backend.server.common.utils.constants import NAME_VALIDATION_REGEX_PATTERN
 
 
 def test_updating_printer() -> None:
@@ -276,7 +277,8 @@ def test_str_data_size():
     assert str_data_size(json.dumps({"hi": "ih"})) == 12
 
 
-def test_prepare_cloudformation_template():
+@pytest.mark.parametrize("is_anyscale_hosted", [True, False])
+def test_prepare_cloudformation_template(is_anyscale_hosted):
 
     mock_region = "us-east-1"
     mock_cfn_stack_name = "mock_cfn_stack_name"
@@ -293,7 +295,11 @@ def test_prepare_cloudformation_template():
 
     with patch("anyscale.util.get_availability_zones", new=Mock(return_value=mock_azs)):
         cfn_template = prepare_cloudformation_template(
-            mock_region, mock_cfn_stack_name, mock_cloud_id, True
+            mock_region,
+            mock_cfn_stack_name,
+            mock_cloud_id,
+            True,
+            is_anyscale_hosted=is_anyscale_hosted,
         )
 
     # If you make some changes to the prepare_cloudformation_template(), update the
@@ -302,10 +308,10 @@ def test_prepare_cloudformation_template():
     # write permission).
     # with open(f"{os.environ['HOME']}/product/frontend/cli/tests/cloud_formation_template.txt", 'w') as f:
     #     f.write(cfn_template)
-
-    with open(
-        f"{os.path.dirname(os.path.abspath(__file__))}/cloud_formation_template.txt"
-    ) as f:
+    cft_name = "cloud_formation_template.txt"
+    if is_anyscale_hosted:
+        cft_name = "cloud_formation_template_oa.txt"
+    with open(f"{os.path.dirname(os.path.abspath(__file__))}/{cft_name}") as f:
         expected_content = f.read()
     assert cfn_template == expected_content
 
@@ -354,84 +360,182 @@ def test_get_ray_and_py_version_for_default_cluster_env(pyversion_param):
         assert ray_version == "mock_latest_ray_version"
 
 
-@pytest.mark.parametrize(
-    "config_dict",
-    [
-        {},
-        {"name": "test-job"},
-        {
-            "name": "test-job",
-            "entrypoint": "python test.py",
-            "build_id": "test_build_id",
-            "compute_config_id": "test_compute_config_id",
-        },
-        {
-            "name": "test-job",
-            "entrypoint": "python test.py",
-            "build_id": "test_build_id",
-            "cloud": "test_cloud_name",
-        },
-    ],
-)
-def test_populate_dict_method(config_dict):
-    original_dict = copy.deepcopy(config_dict)
-    updated_dict = populate_dict_with_workspace_config_if_exists(config_dict, Mock())
-    assert original_dict == updated_dict
+test_workspace_cluster = Mock()
+test_workspace_cluster.name = "workspace-cluster-test_name"
+test_workspace_cluster.project_id = "test_project_id"
+test_workspace_cluster.cluster_environment_build_id = "test_build_id"
+test_workspace_cluster.cluster_compute_id = "test_compute_config_id"
 
 
-@pytest.mark.parametrize(
-    "config_dict",
-    [
-        {"name": "test-job"},
-        {"name": "test-job", "entrypoint": "python test.py"},
-        {
-            "name": "test-job",
-            "entrypoint": "python test.py",
-            "compute_config_id": "user_provided_compute_config_id",
-        },
-        {
-            "name": "test-job",
-            "entrypoint": "python test.py",
-            "build_id": "user_provided_build_id",
-        },
-        {
-            "name": "test-job",
-            "entrypoint": "python test.py",
-            "project_id": "user_provided_project_id",
-            "build_id": "user_provided_build_id",
-            "compute_config_id": "user_provided_compute_config_id",
-        },
-    ],
-)
-def test_populate_dict_method_in_workspace(config_dict: Dict[str, Any]):
-    original_dict = copy.deepcopy(config_dict)
+def test_get_cluster_model_for_current_workspace():
+    api_client_result_mock = Mock()
+    api_client_result_mock.result = test_workspace_cluster
+
     mock_base_api_client = Mock()
-    values_from_cluster = {
-        "project_id": "test_project_id",
-        "cluster_compute_id": "test_compute_config_id",
-        "cluster_environment_build_id": "test_build_id",
-    }
-    parsed_config = {
-        "project_id": "test_project_id",
-        "compute_config_id": "test_compute_config_id",
-        "build_id": "test_build_id",
-    }
-    mock_cluster = Mock(**values_from_cluster)
-    mock_base_api_client.get_cluster = Mock(return_value=Mock(result=mock_cluster))
+    mock_base_api_client.get_cluster = Mock(return_value=api_client_result_mock)
+
+    # If workspace_id or session_id is missing in the env, should return None.
+    assert get_cluster_model_for_current_workspace(mock_base_api_client) is None
+    mock_base_api_client.get_cluster.assert_not_called()
+
+    with patch.dict(
+        os.environ,
+        {"ANYSCALE_EXPERIMENTAL_WORKSPACE_ID": "test_workspace_id",},
+        clear=True,
+    ):
+        assert get_cluster_model_for_current_workspace(mock_base_api_client) is None
+        mock_base_api_client.get_cluster.assert_not_called()
+
+    with patch.dict(
+        os.environ, {"ANYSCALE_SESSION_ID": "test_session_id",}, clear=True,
+    ):
+        assert get_cluster_model_for_current_workspace(mock_base_api_client) is None
+        mock_base_api_client.get_cluster.assert_not_called()
+
+    # If both workspace_id and session_id are present in the env, should call the API
+    # and return the cluster model.
     with patch.dict(
         os.environ,
         {
-            "ANYSCALE_EXPERIMENTAL_WORKSPACE_ID": "Test_workspace_id",
+            "ANYSCALE_EXPERIMENTAL_WORKSPACE_ID": "test_workspace_id",
             "ANYSCALE_SESSION_ID": "test_session_id",
         },
         clear=True,
     ):
-        updated_dict = populate_dict_with_workspace_config_if_exists(
-            config_dict, mock_base_api_client
+        assert (
+            get_cluster_model_for_current_workspace(mock_base_api_client)
+            == test_workspace_cluster
         )
+        mock_base_api_client.get_cluster.assert_called_once_with("test_session_id")
 
-    # The values from the original dict takes priority
-    assert dict(parsed_config, **original_dict) == updated_dict
+
+@pytest.mark.parametrize(
+    ("input_config", "output_config"),
+    [
+        # Test all fields being defaulted.
+        (
+            {"name": "test_job"},
+            {
+                "name": "test_job",
+                "project_id": "test_project_id",
+                "build_id": "test_build_id",
+                "compute_config_id": "test_compute_config_id",
+            },
+        ),
+        # Test no fields being defaulted.
+        (
+            {
+                "name": "test_job",
+                "project_id": "existing_project_id",
+                "build_id": "existing_build_id",
+                "compute_config_id": "existing_compute_config_id",
+            },
+            {
+                "name": "test_job",
+                "project_id": "existing_project_id",
+                "build_id": "existing_build_id",
+                "compute_config_id": "existing_compute_config_id",
+            },
+        ),
+        # If project_id is specified, it shouldn't be overwritten.
+        (
+            {"name": "test_job", "project_id": "existing_project_id"},
+            {
+                "name": "test_job",
+                "project_id": "existing_project_id",
+                "build_id": "test_build_id",
+                "compute_config_id": "test_compute_config_id",
+            },
+        ),
+        # If cluster_env or build_id are specified, they shouldn't be changed.
+        (
+            {"name": "test_job", "cluster_env": "existing_cluster_env"},
+            {
+                "name": "test_job",
+                "project_id": "test_project_id",
+                "cluster_env": "existing_cluster_env",
+                "compute_config_id": "test_compute_config_id",
+            },
+        ),
+        (
+            {"name": "test_job", "build_id": "existing_build_id"},
+            {
+                "name": "test_job",
+                "project_id": "test_project_id",
+                "build_id": "existing_build_id",
+                "compute_config_id": "test_compute_config_id",
+            },
+        ),
+        # If cloud, compute_config, or compute_config_id are specified, they shouldn't be changed.
+        (
+            {"name": "test_job", "cloud": "test_cloud"},
+            {
+                "name": "test_job",
+                "project_id": "test_project_id",
+                "build_id": "test_build_id",
+                "cloud": "test_cloud",
+            },
+        ),
+        (
+            {"name": "test_job", "compute_config": "test_compute_config"},
+            {
+                "name": "test_job",
+                "project_id": "test_project_id",
+                "build_id": "test_build_id",
+                "compute_config": "test_compute_config",
+            },
+        ),
+        (
+            {"name": "test_job", "compute_config_id": "existing_compute_config_id"},
+            {
+                "name": "test_job",
+                "project_id": "test_project_id",
+                "build_id": "test_build_id",
+                "compute_config_id": "existing_compute_config_id",
+            },
+        ),
+    ],
+)
+def test_populate_unspecified_cluster_configs(
+    input_config: Dict[str, str], output_config: Dict[str, str]
+):
+    assert (
+        populate_unspecified_cluster_configs(input_config, test_workspace_cluster)
+        == output_config
+    )
+
+
+def test_populate_unspecified_cluster_configs_populate_name():
+    base_expected_output = {
+        "project_id": "test_project_id",
+        "build_id": "test_build_id",
+        "compute_config_id": "test_compute_config_id",
+    }
+
+    # If a name is passed in, it shouldn't be overwritten.
+    assert populate_unspecified_cluster_configs(
+        {"name": "existing_name"}, test_workspace_cluster, populate_name=True
+    ) == {"name": "existing_name", **base_expected_output}
+
+    # If a name is not passed in and populate_name=False, it shouldn't be overwritten.
+    assert populate_unspecified_cluster_configs({}, test_workspace_cluster,) == {
+        **base_expected_output
+    }
+
+    # If a name is not passed in, it should default to the cluster name stripped of the workspace prefix.
+    assert populate_unspecified_cluster_configs(
+        {}, test_workspace_cluster, populate_name=True
+    ) == {"name": "test_name", **base_expected_output}
+
+    # That that missing the workspace prefix in the cluster name picks it up as-is.
+    test_workspace_cluster_no_name_prefix = Mock()
+    test_workspace_cluster_no_name_prefix.name = "oopsies_unexpected_name"
+    test_workspace_cluster_no_name_prefix.project_id = "test_project_id"
+    test_workspace_cluster_no_name_prefix.cluster_environment_build_id = "test_build_id"
+    test_workspace_cluster_no_name_prefix.cluster_compute_id = "test_compute_config_id"
+    assert populate_unspecified_cluster_configs(
+        {}, test_workspace_cluster_no_name_prefix, populate_name=True
+    ) == {"name": "oopsies_unexpected_name", **base_expected_output}
 
 
 @pytest.mark.parametrize(
@@ -674,7 +778,6 @@ def test_verify_data_plane_role_assume_role_policy(
 
 
 def test_gen_valid_name():
-    NAME_VALIDATION_REGEX_PATTERN = r"^[A-Za-z0-9_-]+$"
     name_without_prefiex = gen_valid_name()
     name_with_prefiex = gen_valid_name("test prefix name")
 
