@@ -1,5 +1,6 @@
 import copy
 import pathlib
+import re
 from typing import List, Optional, Tuple, Union
 
 from common import (
@@ -17,8 +18,10 @@ from anyscale._private.anyscale_client import (
     FakeAnyscaleClient,
     WORKSPACE_CLUSTER_NAME_PREFIX,
 )
+from anyscale._private.models.image_uri import ImageURI
 from anyscale.client.openapi_client.models import (
     ComputeTemplate,
+    ComputeTemplateConfig,
     CreateInternalProductionJob,
     ProductionJobConfig,
 )
@@ -100,7 +103,7 @@ class TestSubmit:
     ):
         sdk, fake_client = sdk_with_fake_client
         fake_client.set_image_uri_mapping(
-            "docker.io/user/my-custom-image:latest", "bld_123"
+            ImageURI.from_str("docker.io/user/my-custom-image:latest"), "bld_123"
         )
 
         config = JobConfig(
@@ -136,6 +139,10 @@ class TestSubmit:
             ComputeTemplate(
                 id="compute_id123",
                 name="my-custom-compute-config",
+                config=ComputeTemplateConfig(
+                    cloud_id="test-cloud-id",
+                    local_vars_configuration=OPENAPI_NO_VALIDATION,
+                ),
                 local_vars_configuration=OPENAPI_NO_VALIDATION,
             )
         )
@@ -194,7 +201,10 @@ class TestSubmit:
         )
 
         # Test an invalid dict.
-        with pytest.raises(RuntimeError, match="Invalid compute config"):
+        with pytest.raises(
+            TypeError,
+            match=re.escape("__init__() got an unexpected keyword argument 'bad'"),
+        ):
             sdk.submit(
                 JobConfig(
                     entrypoint="python hello.py",
@@ -272,6 +282,39 @@ class TestDeployWorkspaceDefaults:
 
 
 class TestOverrideApplicationRuntimeEnvs:
+    @pytest.mark.parametrize(
+        "new_py_modules", [None, [], ["A"], ["C", "D"]],
+    )
+    def test_update_py_modules(
+        self,
+        sdk_with_fake_client: Tuple[JobSDK, FakeAnyscaleClient],
+        new_py_modules: Optional[List[str]],
+    ):
+        sdk, fake_client = sdk_with_fake_client
+        job = JobConfig(
+            name="test-job-name",
+            entrypoint="python hello.py",
+            py_modules=new_py_modules,
+        )
+
+        sdk.submit(job)
+        submitted_runtime_env = fake_client.submitted_job.config.runtime_env
+        if new_py_modules:
+            uploaded_py_modules = [
+                fake_client._upload_uri_mapping[local_dir]
+                for local_dir in new_py_modules
+            ]
+        else:
+            uploaded_py_modules = None
+
+        if uploaded_py_modules:
+            assert submitted_runtime_env.get("py_modules") == uploaded_py_modules
+        else:
+            assert (
+                submitted_runtime_env is None
+                or "py_modules" not in submitted_runtime_env
+            )
+
     @pytest.mark.parametrize("excludes_override", [None, ["override"]])
     @pytest.mark.parametrize(
         "working_dir_override", [None, "./some-local-path", "s3://some-remote-path.zip"]
@@ -287,27 +330,15 @@ class TestOverrideApplicationRuntimeEnvs:
         sdk, fake_client = sdk_with_fake_client
         fake_client.set_inside_workspace(inside_workspace)
 
-        base_job = JobConfig(
+        job = JobConfig(
             name="test-job-name",
             entrypoint="python hello.py",
             working_dir=working_dir_override,
             excludes=excludes_override,
         )
 
-        jobs = [
-            base_job.options(runtime_env=None),
-            base_job.options(runtime_env={}),
-            base_job.options(
-                runtime_env={"working_dir": "other-path/", "excludes": ["existing"],}
-            ),
-            # `None` should avoid workspace overriding.
-            base_job.options(runtime_env={"working_dir": None, "excludes": None,}),
-        ]
-
-        submitted_runtime_envs = []
-        for job in jobs:
-            sdk.submit(job)
-            submitted_runtime_envs.append(fake_client.submitted_job.config.runtime_env)
+        sdk.submit(job)
+        submitted_runtime_env = fake_client.submitted_job.config.runtime_env
 
         cloud_id = (
             FakeAnyscaleClient.WORKSPACE_CLOUD_ID
@@ -317,70 +348,46 @@ class TestOverrideApplicationRuntimeEnvs:
         cwd_uri = fake_client.upload_local_dir_to_cloud_storage(".", cloud_id=cloud_id)
         if working_dir_override is None and not inside_workspace:
             assert (
-                submitted_runtime_envs[0] is None
-                or "working_dir" not in submitted_runtime_envs[0]
+                submitted_runtime_env is None
+                or "working_dir" not in submitted_runtime_env
             )
-            assert "working_dir" not in submitted_runtime_envs[1]
-            assert submitted_runtime_envs[2][
-                "working_dir"
-            ] == fake_client.upload_local_dir_to_cloud_storage(
-                "other-path/", cloud_id=cloud_id,
-            )
-            assert submitted_runtime_envs[3]["working_dir"] is None
         elif working_dir_override is None and inside_workspace:
-            assert submitted_runtime_envs[0]["working_dir"] == cwd_uri
-            assert submitted_runtime_envs[1]["working_dir"] == cwd_uri
-            assert submitted_runtime_envs[2][
-                "working_dir"
-            ] == fake_client.upload_local_dir_to_cloud_storage(
-                "other-path/", cloud_id=cloud_id,
-            )
-            assert submitted_runtime_envs[3]["working_dir"] is None
+            assert submitted_runtime_env["working_dir"] == cwd_uri
         elif working_dir_override is not None and working_dir_override.startswith("s3"):
-            assert submitted_runtime_envs[0]["working_dir"] == working_dir_override
-            assert submitted_runtime_envs[1]["working_dir"] == working_dir_override
-            assert submitted_runtime_envs[2]["working_dir"] == working_dir_override
-            assert submitted_runtime_envs[3]["working_dir"] == working_dir_override
+            assert submitted_runtime_env["working_dir"] == working_dir_override
         else:
-            override_uri = fake_client.upload_local_dir_to_cloud_storage(
+            fake_client.upload_local_dir_to_cloud_storage(
                 "./some-local-path", cloud_id=cloud_id
             )
-            assert submitted_runtime_envs[0]["working_dir"] == override_uri
-            assert submitted_runtime_envs[1]["working_dir"] == override_uri
-            assert submitted_runtime_envs[2]["working_dir"] == override_uri
-            assert submitted_runtime_envs[3]["working_dir"] == override_uri
 
         if excludes_override is None:
             assert (
-                submitted_runtime_envs[0] is None
-                or "excludes" not in submitted_runtime_envs[0]
+                submitted_runtime_env is None or "excludes" not in submitted_runtime_env
             )
-            assert "excludes" not in submitted_runtime_envs[1]
-            assert submitted_runtime_envs[2]["excludes"] == ["existing"]
-            assert submitted_runtime_envs[3]["excludes"] is None
         else:
-            assert submitted_runtime_envs[0]["excludes"] == ["override"]
-            assert submitted_runtime_envs[1]["excludes"] == ["override"]
-            assert submitted_runtime_envs[2]["excludes"] == [
-                "existing",
-                "override",
-            ]
-            assert submitted_runtime_envs[3]["excludes"] == ["override"]
+            assert submitted_runtime_env["excludes"] == ["override"]
 
     @pytest.mark.parametrize(
         "requirements_override",
         [None, MULTI_LINE_REQUIREMENTS.get_path(), ["override"]],
     )
     @pytest.mark.parametrize("workspace_tracking_enabled", [False, True])
+    @pytest.mark.parametrize(
+        "enable_image_build_for_tracked_requirements", [False, True]
+    )
     @pytest.mark.parametrize("inside_workspace", [False, True])
     def test_override_requirements(
         self,
         sdk_with_fake_client: Tuple[JobSDK, FakeAnyscaleClient],
         inside_workspace: bool,
         workspace_tracking_enabled: bool,
+        enable_image_build_for_tracked_requirements: bool,
         requirements_override: Union[None, str, List[str]],
     ):
         sdk, fake_client = sdk_with_fake_client
+        sdk._enable_image_build_for_tracked_requirements = (
+            enable_image_build_for_tracked_requirements
+        )
         fake_client.set_inside_workspace(
             inside_workspace,
             requirements_path=SINGLE_LINE_REQUIREMENTS.get_path()
@@ -388,25 +395,24 @@ class TestOverrideApplicationRuntimeEnvs:
             else None,
         )
 
-        base_job = JobConfig(
+        if inside_workspace and enable_image_build_for_tracked_requirements:
+            fake_client.set_image_uri_mapping(
+                ImageURI.from_str("docker.io/user/my-default-image:latest"),
+                fake_client.WORKSPACE_CLUSTER_ENV_BUILD_ID,
+            )
+            fake_client.set_containerfile_mapping(
+                '# syntax=docker/dockerfile:1\nFROM docker.io/user/my-default-image:latest\nRUN pip install "pip-install-test"',
+                "bld_123",
+            )
+
+        job = JobConfig(
             name="test-job-name",
             entrypoint="python hello.py",
             requirements=requirements_override,
         )
 
-        jobs = [
-            base_job.options(runtime_env=None),
-            base_job.options(runtime_env={}),
-            base_job.options(runtime_env={"pip": MULTI_LINE_REQUIREMENTS.get_path(),}),
-            base_job.options(runtime_env={"pip": ["testabc", "test123"],}),
-            # `None` should avoid workspace overriding.
-            base_job.options(runtime_env={"pip": None,}),
-        ]
-
-        submitted_runtime_envs = []
-        for job in jobs:
-            sdk.submit(job)
-            submitted_runtime_envs.append(fake_client.submitted_job.config.runtime_env)
+        sdk.submit(job)
+        submitted_runtime_env = fake_client.submitted_job.config.runtime_env
 
         if isinstance(requirements_override, str):
             # Override with a file.
@@ -414,30 +420,17 @@ class TestOverrideApplicationRuntimeEnvs:
         elif isinstance(requirements_override, list):
             # Override with a list.
             pass
-        elif inside_workspace and workspace_tracking_enabled:
+        elif (
+            inside_workspace
+            and workspace_tracking_enabled
+            and not enable_image_build_for_tracked_requirements
+        ):
             # Workspace default.
             expected_workspace_pip = SINGLE_LINE_REQUIREMENTS.expected_pip_list
-            assert submitted_runtime_envs[0]["pip"] == expected_workspace_pip
-            assert submitted_runtime_envs[1]["pip"] == expected_workspace_pip
-            assert (
-                submitted_runtime_envs[2]["pip"]
-                == MULTI_LINE_REQUIREMENTS.expected_pip_list
-            )
-            assert submitted_runtime_envs[3]["pip"] == ["testabc", "test123"]
-            assert submitted_runtime_envs[4]["pip"] is None
+            assert submitted_runtime_env["pip"] == expected_workspace_pip
         else:
             # No overrides.
-            assert (
-                submitted_runtime_envs[0] is None
-                or "pip" not in submitted_runtime_envs[0]
-            )
-            assert "pip" not in submitted_runtime_envs[1]
-            assert (
-                submitted_runtime_envs[2]["pip"]
-                == MULTI_LINE_REQUIREMENTS.expected_pip_list
-            )
-            assert submitted_runtime_envs[3]["pip"] == ["testabc", "test123"]
-            assert submitted_runtime_envs[4]["pip"] is None
+            assert submitted_runtime_env is None or "pip" not in submitted_runtime_env
 
 
 class TestDeployUploadDirs:
@@ -475,12 +468,13 @@ class TestDeployUploadDirs:
             ComputeTemplate(
                 id="fake-compute-config-id",
                 name="fake-compute-config",
+                config=ComputeTemplateConfig(
+                    cloud_id="compute-config-cloud-id",
+                    local_vars_configuration=OPENAPI_NO_VALIDATION,
+                ),
                 local_vars_configuration=OPENAPI_NO_VALIDATION,
             )
         )
-        fake_client.set_compute_config_id_to_cloud_id_mapping(
-            "fake-compute-config-id", "compute-config-cloud-id"
-        )
 
         config = JobConfig(
             entrypoint="python test.py",
@@ -513,14 +507,6 @@ class TestDeployUploadDirs:
             "pip": ["pip-install-test"]
         }
 
-        config_with_runtime_env = JobConfig(
-            entrypoint="python test.py", runtime_env={"env_vars": {"foo": "bar"}},
-        )
-        sdk.submit(config_with_runtime_env)
-        assert fake_client.submitted_job.config.runtime_env == {
-            "env_vars": {"foo": "bar"},
-        }
-
     def test_no_upload_remote_working_dir(
         self, sdk_with_fake_client: Tuple[JobSDK, FakeAnyscaleClient]
     ):
@@ -535,125 +521,32 @@ class TestDeployUploadDirs:
             == "s3://some-remote-uri.zip"
         )
 
-    def test_upload_local_py_modules(
-        self, sdk_with_fake_client: Tuple[JobSDK, FakeAnyscaleClient]
-    ):
-        sdk, fake_client = sdk_with_fake_client
-        config = JobConfig(
-            entrypoint="python test.py",
-            runtime_env={
-                "py_modules": [
-                    # Should be left alone.
-                    "s3://some-remote-uri.zip",
-                    # Should be uploaded.
-                    "local-path",
-                ],
-            },
-        )
-        sdk.submit(config)
-        assert (
-            fake_client.submitted_job.config.runtime_env["py_modules"][0]
-            == "s3://some-remote-uri.zip"
-        )
-        assert fake_client.submitted_job.config.runtime_env["py_modules"][1].startswith(
-            FakeAnyscaleClient.CLOUD_BUCKET.format(
-                cloud_id=FakeAnyscaleClient.DEFAULT_CLOUD_ID
-            )
-        )
-
-    def test_upload_caching(
-        self, sdk_with_fake_client: Tuple[JobSDK, FakeAnyscaleClient]
-    ):
-        """The same directory should only by uploaded once."""
-        sdk, fake_client = sdk_with_fake_client
-        config = JobConfig(
-            entrypoint="python test.py",
-            runtime_env={"working_dir": ".", "py_modules": [".", "other-dir"],},
-        )
-        sdk.submit(config)
-        assert fake_client.submitted_job.config.runtime_env["working_dir"].startswith(
-            FakeAnyscaleClient.CLOUD_BUCKET.format(
-                cloud_id=FakeAnyscaleClient.DEFAULT_CLOUD_ID
-            )
-        )
-        assert (
-            fake_client.submitted_job.config.runtime_env["working_dir"]
-            == fake_client.submitted_job.config.runtime_env["py_modules"][0]
-        )
-
-        assert fake_client.submitted_job.config.runtime_env["py_modules"][1].startswith(
-            FakeAnyscaleClient.CLOUD_BUCKET.format(
-                cloud_id=FakeAnyscaleClient.DEFAULT_CLOUD_ID
-            )
-        )
-        assert (
-            fake_client.submitted_job.config.runtime_env["py_modules"][0]
-            != fake_client.submitted_job.config.runtime_env["py_modules"][1]
-        )
-
 
 class TestLoadRequirementsFiles:
     @pytest.mark.parametrize("requirements", [None, *TEST_REQUIREMENTS_FILES])
-    @pytest.mark.parametrize("in_runtime_env", [False, True])
     def test_override_requirements_file(
         self,
         sdk_with_fake_client: Tuple[JobSDK, FakeAnyscaleClient],
         requirements: Optional[RequirementsFile],
-        in_runtime_env: bool,
     ):
         sdk, fake_client = sdk_with_fake_client
 
-        base_job = JobConfig(entrypoint="python hello.py",)
-
-        jobs = [
-            base_job.options(runtime_env=None),
-            base_job.options(runtime_env={}),
-            base_job.options(
-                runtime_env={
-                    "env_vars": {"abc": "123"},
-                    "working_dir": "s3://somewhere.zip",
-                }
-            ),
-        ]
-
-        submitted_runtime_envs = []
-        for i, job in list(enumerate(jobs)):
-            if requirements is not None:
-                if in_runtime_env:
-                    jobs[i] = job.options(
-                        runtime_env={
-                            "pip": requirements.get_path(),
-                            **(job.runtime_env or {}),
-                        },
-                    )
-                else:
-                    jobs[i] = job.options(requirements=requirements.get_path(),)
+        job = JobConfig(entrypoint="python hello.py")
+        if requirements is not None:
+            job = job.options(requirements=requirements.get_path())
 
         if requirements is not None and requirements.expected_pip_list is None:
-            for job in jobs:
-                with pytest.raises(FileNotFoundError):
-                    sdk.submit(job)
+            with pytest.raises(FileNotFoundError):
+                sdk.submit(job)
 
             return
         else:
-            for job in jobs:
-                sdk.submit(job)
-                submitted_runtime_envs.append(
-                    fake_client.submitted_job.config.runtime_env
-                )
+            sdk.submit(job)
 
         if requirements is None:
-            assert submitted_runtime_envs == [job.runtime_env for job in jobs]
+            assert fake_client.submitted_job.config.runtime_env is None
         else:
-            assert submitted_runtime_envs[0] == {
-                "pip": requirements.expected_pip_list,
-            }
-
-            assert submitted_runtime_envs[1] == {
-                "pip": requirements.expected_pip_list,
-            }
-            assert submitted_runtime_envs[2] == {
-                "env_vars": {"abc": "123"},
-                "working_dir": "s3://somewhere.zip",
-                "pip": requirements.expected_pip_list,
-            }
+            assert (
+                fake_client.submitted_job.config.runtime_env["pip"]
+                == requirements.expected_pip_list
+            )
