@@ -8,11 +8,13 @@ import pytest
 
 from anyscale._private.sdk import _LAZY_SDK_SINGLETONS
 from anyscale.commands.service_commands import deploy, status
+from anyscale.compute_config.models import ComputeConfig, HeadNodeConfig
 from anyscale.service.commands import _SERVICE_SDK_SINGLETON_KEY
 from anyscale.service.models import (
     ServiceConfig,
     ServiceState,
     ServiceStatus,
+    ServiceVersionState,
     ServiceVersionStatus,
 )
 
@@ -65,12 +67,14 @@ class FakeServiceSDK:
             query_url="http://fake-service-url/",
             query_auth_token="asdf1234",
             primary_version=ServiceVersionStatus(
-                name="primary",
+                id="primary",
+                state=ServiceVersionState.RUNNING,
                 weight=100,
                 config=ServiceConfig(applications=[{"import_path": "main:app"}]),
             ),
             canary_version=ServiceVersionStatus(
-                name="canary",
+                id="canary",
+                state=ServiceVersionState.STARTING,
                 weight=0,
                 config=ServiceConfig(applications=[{"import_path": "main:app"}]),
             ),
@@ -120,10 +124,7 @@ class TestDeploy:
 
         expected_config = ServiceConfig(
             applications=[
-                {
-                    "import_path": "main:app",
-                    "arguments": {"arg1": "val1", "arg2": "val2"},
-                }
+                {"import_path": "main:app", "args": {"arg1": "val1", "arg2": "val2"},}
             ],
         )
         assert (
@@ -134,7 +135,10 @@ class TestDeploy:
     def test_deploy_from_import_path_with_bad_arg(self, fake_service_sdk):
         runner = CliRunner()
         result = runner.invoke(deploy, ["main:app", "bad_arg"])
-        _assert_error_message(result, message="Invalid application argument 'bad_arg'")
+        _assert_error_message(
+            result,
+            message="Invalid key-value string 'bad_arg'. Must be of the form 'key=value'.",
+        )
 
     def test_deploy_from_file(self, fake_service_sdk):
         runner = CliRunner()
@@ -260,6 +264,16 @@ class TestDeploy:
             == expected_config.applications
         )
 
+    def test_deploy_both_image_uri_and_containerfile(self):
+        runner = CliRunner()
+        result = runner.invoke(
+            deploy, ["main:app", "--image-uri", "image", "--containerfile", "file"]
+        )
+        _assert_error_message(
+            result,
+            message="Only one of '--containerfile' and '--image-uri' can be provided.",
+        )
+
     def test_deploy_from_file_override_options(self, fake_service_sdk):
         runner = CliRunner()
 
@@ -268,17 +282,21 @@ class TestDeploy:
         assert result.exit_code == 0
         assert fake_service_sdk.deployed_config is not None
 
+        inline_compute_config = ComputeConfig(
+            cloud="test-cloud", head_node=HeadNodeConfig(instance_type="m5.2xlarge")
+        )
+
         expected_config = ServiceConfig(
             applications=[{"import_path": "main:app"}],
             name="test-name-from-file",
             image_uri="docker.io/library/test:latest",
-            compute_config="test-compute-config",
+            compute_config=inline_compute_config,
             working_dir="test-working-dir",
             excludes=["test"],
             requirements=["pip-install-test"],
             query_auth_token_enabled=False,
             http_options={"request_timeout_s": 10.0},
-            grpc_options={"grpc_servicer_functions": ["hello.world"],},
+            grpc_options={"grpc_servicer_functions": ["hello.world"]},
             logging_config={"log_level": "DEBUG"},
         )
         assert fake_service_sdk.deployed_config == expected_config
@@ -294,13 +312,13 @@ class TestDeploy:
             applications=[{"import_path": "main:app"}],
             name="override-name",
             image_uri="docker.io/library/test:latest",
-            compute_config="test-compute-config",
+            compute_config=inline_compute_config,
             working_dir="test-working-dir",
             excludes=["test"],
             requirements=["pip-install-test"],
             query_auth_token_enabled=False,
             http_options={"request_timeout_s": 10.0},
-            grpc_options={"grpc_servicer_functions": ["hello.world"],},
+            grpc_options={"grpc_servicer_functions": ["hello.world"]},
             logging_config={"log_level": "DEBUG"},
         )
         assert fake_service_sdk.deployed_config == expected_config
@@ -322,13 +340,13 @@ class TestDeploy:
             applications=[{"import_path": "main:app"}],
             name="test-name-from-file",
             image_uri="docker.io/user/override-image:latest",
-            compute_config="test-compute-config",
+            compute_config=inline_compute_config,
             working_dir="test-working-dir",
             excludes=["test"],
             requirements=["pip-install-test"],
             query_auth_token_enabled=False,
             http_options={"request_timeout_s": 10.0},
-            grpc_options={"grpc_servicer_functions": ["hello.world"],},
+            grpc_options={"grpc_servicer_functions": ["hello.world"]},
             logging_config={"log_level": "DEBUG"},
         )
         assert fake_service_sdk.deployed_config == expected_config
@@ -351,7 +369,7 @@ class TestDeploy:
             requirements=["pip-install-test"],
             query_auth_token_enabled=False,
             http_options={"request_timeout_s": 10.0},
-            grpc_options={"grpc_servicer_functions": ["hello.world"],},
+            grpc_options={"grpc_servicer_functions": ["hello.world"]},
             logging_config={"log_level": "DEBUG"},
         )
         assert fake_service_sdk.deployed_config == expected_config
@@ -367,13 +385,13 @@ class TestDeploy:
             applications=[{"import_path": "main:app"}],
             name="test-name-from-file",
             image_uri="docker.io/library/test:latest",
-            compute_config="test-compute-config",
+            compute_config=inline_compute_config,
             working_dir="override-working-dir",
             excludes=["test"],
             requirements=["pip-install-test"],
             query_auth_token_enabled=False,
             http_options={"request_timeout_s": 10.0},
-            grpc_options={"grpc_servicer_functions": ["hello.world"],},
+            grpc_options={"grpc_servicer_functions": ["hello.world"]},
             logging_config={"log_level": "DEBUG"},
         )
         assert fake_service_sdk.deployed_config == expected_config
@@ -390,17 +408,48 @@ class TestDeploy:
             applications=[{"import_path": "main:app"}],
             name="test-name-from-file",
             image_uri="docker.io/library/test:latest",
-            compute_config="test-compute-config",
+            compute_config=inline_compute_config,
             working_dir="test-working-dir",
             excludes=["test"],
             requirements=MULTI_LINE_REQUIREMENTS_PATH,
             query_auth_token_enabled=False,
             http_options={"request_timeout_s": 10.0},
-            grpc_options={"grpc_servicer_functions": ["hello.world"],},
+            grpc_options={"grpc_servicer_functions": ["hello.world"]},
             logging_config={"log_level": "DEBUG"},
         )
         assert fake_service_sdk.deployed_config == expected_config
 
+        # Override environment variables.
+        result = runner.invoke(
+            deploy,
+            [
+                "-f",
+                FULL_CONFIG_PATH,
+                "--env",
+                "FOO=BAR_override",
+                "--env",
+                "FOO2=BAR2",
+            ],
+        )
+        print(result.stdout)
+        assert result.exit_code == 0
+        assert fake_service_sdk.deployed_config is not None
+        expected_config = ServiceConfig(
+            applications=[{"import_path": "main:app"}],
+            name="test-name-from-file",
+            image_uri="docker.io/library/test:latest",
+            compute_config=inline_compute_config,
+            working_dir="test-working-dir",
+            excludes=["test"],
+            requirements=["pip-install-test"],
+            query_auth_token_enabled=False,
+            http_options={"request_timeout_s": 10.0},
+            grpc_options={"grpc_servicer_functions": ["hello.world"]},
+            logging_config={"log_level": "DEBUG"},
+            env_vars={"FOO": "BAR_override", "FOO2": "BAR2"},
+        )
+        assert fake_service_sdk.deployed_config == expected_config
+
 
 class TestStatus:
     def test_no_name(self, fake_service_sdk):
@@ -468,8 +517,8 @@ class TestStatus:
         assert fake_service_sdk.fetched_name == "custom-name"
         assert "primary_version" in result.stdout
         assert "canary_version" in result.stdout
-        assert "query_url" not in result.stdout
-        assert "query_auth_token" not in result.stdout
+        assert "query_url" in result.stdout
+        assert "query_auth_token" in result.stdout
         assert "config" not in result.stdout
         assert "applications" not in result.stdout
 
