@@ -1,6 +1,7 @@
 import copy
 from dataclasses import dataclass
 import pathlib
+import re
 from typing import Any, Dict, List, Optional, Tuple, Union
 
 from common import (
@@ -19,21 +20,31 @@ from anyscale._private.anyscale_client import (
     FakeAnyscaleClient,
     WORKSPACE_CLUSTER_NAME_PREFIX,
 )
-from anyscale.client.openapi_client.models import ComputeTemplate, ComputeTemplateConfig
+from anyscale._private.models.image_uri import ImageURI
+from anyscale.client.openapi_client.models import (
+    Cloud,
+    ComputeNodeType,
+    ComputeTemplate,
+    ComputeTemplateConfig,
+)
+from anyscale.compute_config import ComputeConfig, HeadNodeConfig
 from anyscale.sdk.anyscale_client.models import (
     AccessConfig,
     ApplyServiceModel,
     ProductionServiceV2VersionModel,
-    RayGCSExternalStorageConfig,
+    RayGCSExternalStorageConfig as ExternalAPIRayGCSExternalStorageConfig,
     ServiceConfig as ExternalAPIServiceConfig,
     ServiceEventCurrentState,
     ServiceModel,
+    ServiceVersionState as ExternalAPIServiceVersionState,
 )
 from anyscale.service._private.service_sdk import ServiceSDK
 from anyscale.service.models import (
+    RayGCSExternalStorageConfig,
     ServiceConfig,
     ServiceState,
     ServiceStatus,
+    ServiceVersionState,
     ServiceVersionStatus,
 )
 from anyscale.utils.workspace_notification import (
@@ -130,7 +141,7 @@ class TestDeploy:
     ):
         sdk, fake_client, _ = sdk_with_fake_client
         fake_client.set_image_uri_mapping(
-            "docker.io/user/my-custom-image:latest", "bld_123"
+            ImageURI.from_str("docker.io/user/my-custom-image:latest"), "bld_123"
         )
 
         config = ServiceConfig(
@@ -166,6 +177,10 @@ class TestDeploy:
             ComputeTemplate(
                 id="compute_id123",
                 name="my-custom-compute-config",
+                config=ComputeTemplateConfig(
+                    cloud_id="test-cloud-id",
+                    local_vars_configuration=OPENAPI_NO_VALIDATION,
+                ),
                 local_vars_configuration=OPENAPI_NO_VALIDATION,
             )
         )
@@ -223,7 +238,10 @@ class TestDeploy:
         )
 
         # Test an invalid dict.
-        with pytest.raises(RuntimeError, match="Invalid compute config"):
+        with pytest.raises(
+            TypeError,
+            match=re.escape("__init__() got an unexpected keyword argument 'bad'"),
+        ):
             config = ServiceConfig(
                 applications=[{"import_path": "main:app"}],
                 name="test-service-name",
@@ -231,6 +249,73 @@ class TestDeploy:
             )
             sdk.deploy(config)
 
+    def test_reject_changed_cloud(
+        self, sdk_with_fake_client: Tuple[ServiceSDK, FakeAnyscaleClient, TestLogger]
+    ):
+        sdk, fake_client, _ = sdk_with_fake_client
+
+        fake_client.add_cloud(
+            Cloud(
+                id="test-cloud-id",
+                name="test-cloud",
+                local_vars_configuration=OPENAPI_NO_VALIDATION,
+            )
+        )
+        fake_client.add_compute_config(
+            ComputeTemplate(
+                id="compute_id123",
+                name="my-custom-compute-config",
+                config=ComputeTemplateConfig(
+                    cloud_id="test-cloud-id",
+                    local_vars_configuration=OPENAPI_NO_VALIDATION,
+                ),
+                local_vars_configuration=OPENAPI_NO_VALIDATION,
+            )
+        )
+
+        fake_client.add_cloud(
+            Cloud(
+                id="test-other-cloud-id",
+                name="test-other-cloud",
+                local_vars_configuration=OPENAPI_NO_VALIDATION,
+            )
+        )
+        fake_client.add_compute_config(
+            ComputeTemplate(
+                id="compute_id456",
+                name="my-other-compute-config",
+                config=ComputeTemplateConfig(
+                    cloud_id="test-other-cloud-id",
+                    local_vars_configuration=OPENAPI_NO_VALIDATION,
+                ),
+                local_vars_configuration=OPENAPI_NO_VALIDATION,
+            )
+        )
+
+        config = ServiceConfig(
+            applications=[{"import_path": "main:app"}],
+            name="test-service-name",
+            compute_config="my-custom-compute-config",
+        )
+        sdk.deploy(config)
+        assert fake_client.rolled_out_model == ApplyServiceModel(
+            name="test-service-name",
+            project_id=fake_client.DEFAULT_PROJECT_ID,
+            build_id=fake_client.DEFAULT_CLUSTER_ENV_BUILD_ID,
+            compute_config_id="compute_id123",
+            ray_serve_config={"applications": config.applications},
+            rollout_strategy="ROLLOUT",
+            config=ExternalAPIServiceConfig(
+                access=AccessConfig(use_bearer_token=True),
+            ),
+        )
+
+        with pytest.raises(
+            ValueError,
+            match="The cloud for a service cannot be changed once it's created. Service 'test-service-name' was created on cloud 'test-cloud', but the provided config is for cloud 'test-other-cloud'.",
+        ):
+            sdk.deploy(config.options(compute_config="my-other-compute-config"))
+
     def test_http_options(
         self, sdk_with_fake_client: Tuple[ServiceSDK, FakeAnyscaleClient, TestLogger]
     ):
@@ -317,19 +402,161 @@ class TestDeploy:
         config = ServiceConfig(
             applications=[{"import_path": "main:app"}], name="test-service-name"
         )
+
+        sdk.deploy(config, in_place=False)
+        assert fake_client.rolled_out_model == ApplyServiceModel(
+            name="test-service-name",
+            project_id=fake_client.DEFAULT_PROJECT_ID,
+            build_id=fake_client.DEFAULT_CLUSTER_ENV_BUILD_ID,
+            compute_config_id=fake_client.DEFAULT_CLUSTER_COMPUTE_ID,
+            ray_serve_config={"applications": [{"import_path": "main:app"}]},
+            rollout_strategy="ROLLOUT",
+            config=ExternalAPIServiceConfig(
+                access=AccessConfig(use_bearer_token=True),
+            ),
+        )
+
+        config = config.options(applications=[{"import_path": "other:app"}])
         sdk.deploy(config, in_place=True)
         assert fake_client.rolled_out_model == ApplyServiceModel(
             name="test-service-name",
             project_id=fake_client.DEFAULT_PROJECT_ID,
             build_id=fake_client.DEFAULT_CLUSTER_ENV_BUILD_ID,
             compute_config_id=fake_client.DEFAULT_CLUSTER_COMPUTE_ID,
-            ray_serve_config={"applications": config.applications},
+            ray_serve_config={"applications": [{"import_path": "other:app"}]},
             rollout_strategy="IN_PLACE",
             config=ExternalAPIServiceConfig(
                 access=AccessConfig(use_bearer_token=True),
             ),
         )
 
+        # Cluster-level options should be ignored, but application-level options should be applied.
+        config = config.options(
+            # These should be applied.
+            applications=[{"import_path": "yet_other:app"}],
+            working_dir="s3://remote.zip",
+            requirements=["test"],
+            logging_config={"log_level": "DEBUG"},
+            # These should all be ignored.
+            image_uri="docker.io/user/my-custom-image:latest",
+            compute_config="foobar:123",
+            ray_gcs_external_storage_config=RayGCSExternalStorageConfig(
+                address="test.com:8000"
+            ),
+            query_auth_token_enabled=False,
+        )
+        sdk.deploy(config, in_place=True)
+        assert fake_client.rolled_out_model == ApplyServiceModel(
+            name="test-service-name",
+            project_id=fake_client.DEFAULT_PROJECT_ID,
+            build_id=fake_client.DEFAULT_CLUSTER_ENV_BUILD_ID,
+            compute_config_id=fake_client.DEFAULT_CLUSTER_COMPUTE_ID,
+            ray_serve_config={
+                "applications": [
+                    {
+                        "import_path": "yet_other:app",
+                        "runtime_env": {
+                            "working_dir": "s3://remote.zip",
+                            "pip": ["test"],
+                        },
+                    }
+                ],
+                "logging_config": {"log_level": "DEBUG"},
+            },
+            rollout_strategy="IN_PLACE",
+            config=ExternalAPIServiceConfig(
+                access=AccessConfig(use_bearer_token=True),
+            ),
+        )
+
+    def test_in_place_service_does_not_exist(
+        self, sdk_with_fake_client: Tuple[ServiceSDK, FakeAnyscaleClient, TestLogger]
+    ):
+        sdk, fake_client, _ = sdk_with_fake_client
+
+        config = ServiceConfig(
+            applications=[{"import_path": "main:app"}], name="test-service-name"
+        )
+
+        # in_place should fail if the service doesn't already exist.
+        with pytest.raises(
+            RuntimeError,
+            match="in_place can only be used to update running services, but no service was found with name 'test-service-name'",
+        ):
+            sdk.deploy(config, in_place=True)
+
+    def test_in_place_service_terminated(
+        self, sdk_with_fake_client: Tuple[ServiceSDK, FakeAnyscaleClient, TestLogger]
+    ):
+        sdk, fake_client, _ = sdk_with_fake_client
+        config = ServiceConfig(
+            applications=[{"import_path": "main:app"}], name="test-service-name"
+        )
+        sdk.deploy(config)
+
+        sdk.terminate(name="test-service-name")
+        assert sdk.status(name="test-service-name").state == ServiceState.TERMINATED
+
+        with pytest.raises(
+            RuntimeError,
+            match="in_place can only be used to update running services, but service 'test-service-name' is terminated",
+        ):
+            sdk.deploy(config, in_place=True)
+
+    def test_in_place_during_rollout(
+        self, sdk_with_fake_client: Tuple[ServiceSDK, FakeAnyscaleClient, TestLogger]
+    ):
+        sdk, fake_client, _ = sdk_with_fake_client
+
+        # Service must exist before in_place is allowed.
+        config = ServiceConfig(
+            applications=[{"import_path": "main:app"}], name="test-service-name"
+        )
+        sdk.deploy(config, in_place=False)
+
+        # Update the service to mimic an ongoing rollout (canary_version is populated).
+        service = fake_client.get_service(name="test-service-name")
+        service.canary_version = ProductionServiceV2VersionModel(
+            version="canary",
+            current_state=ExternalAPIServiceVersionState.STARTING,
+            weight=0,
+            build_id=service.primary_version.build_id,
+            compute_config_id=service.primary_version.compute_config_id,
+            ray_serve_config=service.primary_version.ray_serve_config,
+            ray_gcs_external_storage_config=service.primary_version.ray_gcs_external_storage_config,
+            local_vars_configuration=OPENAPI_NO_VALIDATION,
+        )
+
+        with pytest.raises(
+            RuntimeError,
+            match="in_place updates cannot be used while a rollout is in progress. Complete the rollout or roll back first.",
+        ):
+            sdk.deploy(config, in_place=True)
+
+    def test_in_place_rejects_rollout_options(
+        self, sdk_with_fake_client: Tuple[ServiceSDK, FakeAnyscaleClient, TestLogger]
+    ):
+        sdk, fake_client, _ = sdk_with_fake_client
+
+        # Service must exist before in_place is allowed.
+        config = ServiceConfig(
+            applications=[{"import_path": "main:app"}], name="test-service-name"
+        )
+        sdk.deploy(config, in_place=False)
+
+        # in_place does not support canary_percent or max_surge_percent.
+        with pytest.raises(
+            ValueError,
+            match="canary_percent cannot be specified when doing an in_place update",
+        ):
+            sdk.deploy(config, in_place=True, canary_percent=50)
+
+        with pytest.raises(
+            ValueError,
+            match="max_surge_percent cannot be specified when doing an in_place update",
+        ):
+            sdk.deploy(config, in_place=True, max_surge_percent=50)
+
     def test_canary_percent(
         self, sdk_with_fake_client: Tuple[ServiceSDK, FakeAnyscaleClient, TestLogger]
     ):
@@ -407,7 +634,9 @@ class TestDeploy:
         config = ServiceConfig(
             applications=[{"import_path": "main:app"}],
             name="test-service-name",
-            ray_gcs_external_storage_config={"address": "test.com:8000"},
+            ray_gcs_external_storage_config=RayGCSExternalStorageConfig(
+                address="test.com:8000"
+            ),
         )
         sdk.deploy(config)
         assert fake_client.rolled_out_model == ApplyServiceModel(
@@ -420,7 +649,7 @@ class TestDeploy:
             config=ExternalAPIServiceConfig(
                 access=AccessConfig(use_bearer_token=True),
             ),
-            ray_gcs_external_storage_config=RayGCSExternalStorageConfig(
+            ray_gcs_external_storage_config=ExternalAPIRayGCSExternalStorageConfig(
                 enable=True,
                 address="test.com:8000",
                 redis_certificate_path="/etc/ssl/certs/ca-certificates.crt",
@@ -540,6 +769,81 @@ class TestDeployWorkspaceDefaults:
 
 
 class TestOverrideApplicationRuntimeEnvs:
+    @pytest.mark.parametrize(
+        "new_py_modules", [None, [], ["A"], ["C", "D"]],
+    )
+    @pytest.mark.parametrize("existing_py_modules", [None, [], ["A"], ["B"]])
+    def test_update_py_modules(
+        self,
+        sdk_with_fake_client: Tuple[ServiceSDK, FakeAnyscaleClient, TestLogger],
+        new_py_modules: Optional[List[str]],
+        existing_py_modules: Optional[List[str]],
+    ):
+        sdk, fake_client, _ = sdk_with_fake_client
+        app: Dict[str, Any] = {"import_path": "main:app"}
+        if existing_py_modules:
+            app["runtime_env"] = {"py_modules": existing_py_modules}
+        config = ServiceConfig(
+            applications=[app], name="test-service-name", py_modules=new_py_modules,
+        )
+        sdk.deploy(config)
+        applications = fake_client.rolled_out_model.ray_serve_config["applications"]
+
+        if new_py_modules or existing_py_modules:
+            if existing_py_modules is None:
+                existing_py_modules = []
+            if new_py_modules is None:
+                new_py_modules = []
+            uploaded_py_modules = [
+                fake_client._upload_uri_mapping[local_dir]
+                for local_dir in existing_py_modules + new_py_modules
+            ]
+        else:
+            uploaded_py_modules = None
+
+        if uploaded_py_modules:
+            for app in applications:
+                assert app["runtime_env"].get("py_modules") == uploaded_py_modules
+        else:
+            assert "py_modules" not in app.get("runtime_env", {})
+
+    @pytest.mark.parametrize(
+        "new_env_vars",
+        [None, {}, {"A": "B"}, {"C": "D", "E": "F"}, {"K": "V1", "C": "D"}],
+    )
+    @pytest.mark.parametrize("existing_env_vars", [None, {}, {"K": "V"}, {"A": "B"}])
+    @pytest.mark.parametrize("inside_workspace", [False, True])
+    def test_update_env_vars(
+        self,
+        sdk_with_fake_client: Tuple[ServiceSDK, FakeAnyscaleClient, TestLogger],
+        new_env_vars: Optional[Dict[str, str]],
+        existing_env_vars: Optional[Dict[str, str]],
+        inside_workspace: bool,
+    ):
+        sdk, fake_client, _ = sdk_with_fake_client
+        fake_client.set_inside_workspace(inside_workspace)
+        app: Dict[str, Any] = {"import_path": "main:app"}
+        if existing_env_vars:
+            app["runtime_env"] = {"env_vars": existing_env_vars}
+
+        config = ServiceConfig(
+            applications=[app], name="test-service-name", env_vars=new_env_vars,
+        )
+        sdk.deploy(config)
+        applications = fake_client.rolled_out_model.ray_serve_config["applications"]
+        for app in applications:
+            if new_env_vars or existing_env_vars:
+                if existing_env_vars is None:
+                    existing_env_vars = {}
+                if new_env_vars is None:
+                    new_env_vars = {}
+                assert app["runtime_env"].get("env_vars") == {
+                    **existing_env_vars,
+                    **new_env_vars,
+                }
+            else:
+                assert "env_vars" not in app.get("runtime_env", {})
+
     @pytest.mark.parametrize("excludes_override", [None, ["override"]])
     @pytest.mark.parametrize(
         "working_dir_override", [None, "./some-local-path", "s3://some-remote-path.zip"]
@@ -640,6 +944,9 @@ class TestOverrideApplicationRuntimeEnvs:
             ]
             assert applications[3]["runtime_env"]["excludes"] == ["override"]
 
+    @pytest.mark.parametrize(
+        "enable_image_build_for_tracked_requirements", [False, True]
+    )
     @pytest.mark.parametrize(
         "requirements_override",
         [None, MULTI_LINE_REQUIREMENTS.get_path(), ["override"]],
@@ -652,6 +959,7 @@ class TestOverrideApplicationRuntimeEnvs:
         inside_workspace: bool,
         workspace_tracking_enabled: bool,
         requirements_override: Union[None, str, List[str]],
+        enable_image_build_for_tracked_requirements: bool,
     ):
         sdk, fake_client, _ = sdk_with_fake_client
         fake_client.set_inside_workspace(
@@ -660,6 +968,18 @@ class TestOverrideApplicationRuntimeEnvs:
             if workspace_tracking_enabled
             else None,
         )
+        sdk._enable_image_build_for_tracked_requirements = (
+            enable_image_build_for_tracked_requirements
+        )
+        if inside_workspace and enable_image_build_for_tracked_requirements:
+            fake_client.set_image_uri_mapping(
+                ImageURI.from_str("docker.io/user/my-default-image:latest"),
+                fake_client.WORKSPACE_CLUSTER_ENV_BUILD_ID,
+            )
+            fake_client.set_containerfile_mapping(
+                '# syntax=docker/dockerfile:1\nFROM docker.io/user/my-default-image:latest\nRUN pip install "pip-install-test"',
+                "bld_123",
+            )
 
         config = ServiceConfig(
             name="test-service-name",
@@ -700,7 +1020,11 @@ class TestOverrideApplicationRuntimeEnvs:
         elif isinstance(requirements_override, list):
             # Override with a list.
             pass
-        elif inside_workspace and workspace_tracking_enabled:
+        elif (
+            inside_workspace
+            and workspace_tracking_enabled
+            and not enable_image_build_for_tracked_requirements
+        ):
             # Workspace default.
             expected_workspace_pip = SINGLE_LINE_REQUIREMENTS.expected_pip_list
             assert applications[0]["runtime_env"]["pip"] == expected_workspace_pip
@@ -765,11 +1089,12 @@ class TestDeployUploadDirs:
                 id="fake-compute-config-id",
                 name="fake-compute-config",
                 local_vars_configuration=OPENAPI_NO_VALIDATION,
+                config=ComputeTemplateConfig(
+                    cloud_id="compute-config-cloud-id",
+                    local_vars_configuration=OPENAPI_NO_VALIDATION,
+                ),
             )
         )
-        fake_client.set_compute_config_id_to_cloud_id_mapping(
-            "fake-compute-config-id", "compute-config-cloud-id"
-        )
 
         config = ServiceConfig(
             applications=[{"import_path": "main:app"}],
@@ -1032,9 +1357,10 @@ def _expected_status(**kwargs) -> ServiceStatus:
 
 
 def _service_version_model(
-    name: str,
+    id: str,  # noqa: A002
     weight: int,
     *,
+    state: ExternalAPIServiceVersionState = ExternalAPIServiceVersionState.RUNNING,
     build_id: str = TEST_CLUSTER_ENV_BUILD_ID,
     compute_config_id: str = TEST_COMPUTE_CONFIG_ID,
     applications: Optional[List[Dict[str, Any]]] = None,
@@ -1054,12 +1380,13 @@ def _service_version_model(
         ray_serve_config["logging_config"] = logging_config
 
     return ProductionServiceV2VersionModel(
-        version=name,
+        id=id,
+        current_state=state,
         weight=weight,
         build_id=build_id,
         compute_config_id=compute_config_id,
         ray_serve_config=ray_serve_config,
-        ray_gcs_external_storage_config=RayGCSExternalStorageConfig(
+        ray_gcs_external_storage_config=ExternalAPIRayGCSExternalStorageConfig(
             **ray_gcs_external_storage_config
         )
         if ray_gcs_external_storage_config
@@ -1149,7 +1476,8 @@ SERVICE_STATUS_TEST_CASES = [
         expected_status=_expected_status(
             state=ServiceState.RUNNING,
             primary_version=ServiceVersionStatus(
-                name="primary",
+                id="primary",
+                state=ServiceVersionState.RUNNING,
                 weight=100,
                 config=ServiceConfig(
                     name=TEST_SERVICE_NAME,
@@ -1165,12 +1493,17 @@ SERVICE_STATUS_TEST_CASES = [
         state=ServiceEventCurrentState.RUNNING,
         primary_version=_service_version_model("primary", 50),
         canary_version=_service_version_model(
-            "canary", 50, http_options=TEST_HTTP_OPTIONS, grpc_options=TEST_GRPC_OPTIONS
+            "canary",
+            0,
+            http_options=TEST_HTTP_OPTIONS,
+            grpc_options=TEST_GRPC_OPTIONS,
+            state=ExternalAPIServiceVersionState.STARTING,
         ),
         expected_status=_expected_status(
             state=ServiceState.RUNNING,
             primary_version=ServiceVersionStatus(
-                name="primary",
+                id="primary",
+                state=ServiceVersionState.RUNNING,
                 weight=50,
                 config=ServiceConfig(
                     name=TEST_SERVICE_NAME,
@@ -1181,8 +1514,9 @@ SERVICE_STATUS_TEST_CASES = [
                 ),
             ),
             canary_version=ServiceVersionStatus(
-                name="canary",
-                weight=50,
+                id="canary",
+                state=ServiceVersionState.STARTING,
+                weight=0,
                 config=ServiceConfig(
                     name=TEST_SERVICE_NAME,
                     applications=TEST_APPLICATIONS,
@@ -1323,7 +1657,15 @@ class TestStatus:
                 anonymous=True,
                 id="anonymous-compute-config-id",
                 name="anonymous-compute-config-name",
-                config=ComputeTemplateConfig(**TEST_COMPUTE_CONFIG_DICT),
+                config=ComputeTemplateConfig(
+                    cloud_id=fake_client.DEFAULT_CLOUD_ID,
+                    head_node_type=ComputeNodeType(
+                        name="head-node", instance_type="head-node-instance-type",
+                    ),
+                    worker_node_types=[],
+                    auto_select_worker_config=True,
+                    local_vars_configuration=OPENAPI_NO_VALIDATION,
+                ),
                 local_vars_configuration=OPENAPI_NO_VALIDATION,
             )
         )
@@ -1336,18 +1678,19 @@ class TestStatus:
             expected_status=_expected_status(
                 state=ServiceState.RUNNING,
                 primary_version=ServiceVersionStatus(
-                    name="primary",
+                    id="primary",
+                    state=ServiceVersionState.RUNNING,
                     weight=100,
                     config=ServiceConfig(
                         name=TEST_SERVICE_NAME,
                         applications=TEST_APPLICATIONS,
                         image_uri=TEST_IMAGE_URI,
-                        compute_config={
-                            # Internal fields like cloud_id should be stripped.
-                            k: v
-                            for k, v in TEST_COMPUTE_CONFIG_DICT.items()
-                            if k != "cloud_id"
-                        },
+                        compute_config=ComputeConfig(
+                            cloud=fake_client.DEFAULT_CLOUD_NAME,
+                            head_node=HeadNodeConfig(
+                                instance_type="head-node-instance-type"
+                            ),
+                        ),
                         query_auth_token_enabled=False,
                     ),
                 ),
@@ -1385,7 +1728,8 @@ class TestStatus:
             expected_status=_expected_status(
                 state=ServiceState.RUNNING,
                 primary_version=ServiceVersionStatus(
-                    name="primary",
+                    id="primary",
+                    state=ServiceVersionState.RUNNING,
                     weight=100,
                     config=ServiceConfig(
                         name=TEST_SERVICE_NAME,
@@ -1393,11 +1737,11 @@ class TestStatus:
                         image_uri=TEST_IMAGE_URI,
                         compute_config="compute-config-name",
                         query_auth_token_enabled=False,
-                        ray_gcs_external_storage_config={
-                            "address": "host:1234",
-                            "enable": True,
-                            "redis_certificate_path": "/etc/ssl/certs/ca-certificates.crt",
-                        },
+                        ray_gcs_external_storage_config=RayGCSExternalStorageConfig(
+                            enabled=True,
+                            address="host:1234",
+                            certificate_path="/etc/ssl/certs/ca-certificates.crt",
+                        ),
                     ),
                 ),
             ),
@@ -1434,7 +1778,8 @@ class TestStatus:
             expected_status=_expected_status(
                 state=ServiceState.RUNNING,
                 primary_version=ServiceVersionStatus(
-                    name="primary",
+                    id="primary",
+                    state=ServiceVersionState.RUNNING,
                     weight=100,
                     config=ServiceConfig(
                         name=TEST_SERVICE_NAME,
@@ -1478,7 +1823,8 @@ class TestStatus:
             expected_status=_expected_status(
                 state=ServiceState.RUNNING,
                 primary_version=ServiceVersionStatus(
-                    name="primary",
+                    id="primary",
+                    state=ServiceVersionState.RUNNING,
                     weight=100,
                     config=ServiceConfig(
                         name=TEST_SERVICE_NAME,
@@ -1515,7 +1861,8 @@ class TestStatus:
             expected_status=_expected_status(
                 state=ServiceState.RUNNING,
                 primary_version=ServiceVersionStatus(
-                    name="primary",
+                    id="primary",
+                    state=ServiceVersionState.RUNNING,
                     weight=100,
                     config=ServiceConfig(
                         name=TEST_SERVICE_NAME,
